
class Armour:
    CLOTH_PANTS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Cloth/PrimalItemArmor_ClothPants.PrimalItemArmor_ClothPants'"
    CLOTH_SHIRT = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Cloth/PrimalItemArmor_ClothShirt.PrimalItemArmor_ClothShirt'"
    CLOTH_HAT = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Cloth/PrimalItemArmor_ClothHelmet.PrimalItemArmor_ClothHelmet'"
    CLOTH_BOOTS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Cloth/PrimalItemArmor_ClothBoots.PrimalItemArmor_ClothBoots'"
    CLOTH_GLOVES = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Cloth/PrimalItemArmor_ClothGloves.PrimalItemArmor_ClothGloves'"
    HIDE_PANTS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Leather/PrimalItemArmor_HidePants.PrimalItemArmor_HidePants'"
    HIDE_SHIRT = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Leather/PrimalItemArmor_HideShirt.PrimalItemArmor_HideShirt'"
    HIDE_HAT = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Leather/PrimalItemArmor_HideHelmet.PrimalItemArmor_HideHelmet'"
    HIDE_BOOTS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Leather/PrimalItemArmor_HideBoots.PrimalItemArmor_HideBoots'"
    HIDE_GLOVES = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Leather/PrimalItemArmor_HideGloves.PrimalItemArmor_HideGloves'"
    CHITIN_LEGGINGS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Chitin/PrimalItemArmor_ChitinPants.PrimalItemArmor_ChitinPants'"
    CHITIN_CHESTPIECE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Chitin/PrimalItemArmor_ChitinShirt.PrimalItemArmor_ChitinShirt'"
    CHITIN_HELMET = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Chitin/PrimalItemArmor_ChitinHelmet.PrimalItemArmor_ChitinHelmet'"
    CHITIN_BOOTS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Chitin/PrimalItemArmor_ChitinBoots.PrimalItemArmor_ChitinBoots'"
    CHITIN_GAUNTLETS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Chitin/PrimalItemArmor_ChitinGloves.PrimalItemArmor_ChitinGloves'"
    PARACHUTE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Consumables/BaseBPs/PrimalItemConsumableBuff_Parachute.PrimalItemConsumableBuff_Parachute'"
    FLAK_LEGGINGS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Metal/PrimalItemArmor_MetalPants.PrimalItemArmor_MetalPants'"
    FLAK_CHESTPIECE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Metal/PrimalItemArmor_MetalShirt.PrimalItemArmor_MetalShirt'"
    FLAK_HELMET = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Metal/PrimalItemArmor_MetalHelmet.PrimalItemArmor_MetalHelmet'"
    FLAK_BOOTS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Metal/PrimalItemArmor_MetalBoots.PrimalItemArmor_MetalBoots'"
    FLAK_GAUNTLETS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Metal/PrimalItemArmor_MetalGloves.PrimalItemArmor_MetalGloves'"
    GLIDER_SUIT = "Blueprint'/Game/Aberration/CoreBlueprints/Items/Armor/PrimalItemArmor_Glider.PrimalItemArmor_Glider'"
    HEAVY_MINERS_HELMET = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Metal/PrimalItemArmor_MinersHelmet.PrimalItemArmor_MinersHelmet'"
    SCUBA_TANK = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/SCUBA/PrimalItemArmor_ScubaShirt_SuitWithTank.PrimalItemArmor_ScubaShirt_SuitWithTank'"
    SCUBA_MASK = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/SCUBA/PrimalItemArmor_ScubaHelmet_Goggles.PrimalItemArmor_ScubaHelmet_Goggles'"
    SCUBA_FLIPPERS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/SCUBA/PrimalItemArmor_ScubaBoots_Flippers.PrimalItemArmor_ScubaBoots_Flippers'"
    FUR_LEGGINGS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Fur/PrimalItemArmor_FurPants.PrimalItemArmor_FurPants'"
    FUR_CHESTPIECE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Fur/PrimalItemArmor_FurShirt.PrimalItemArmor_FurShirt'"
    FUR_CAP = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Fur/PrimalItemArmor_FurHelmet.PrimalItemArmor_FurHelmet'"
    FUR_BOOTS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Fur/PrimalItemArmor_FurBoots.PrimalItemArmor_FurBoots'"
    FUR_GAUNTLETS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Fur/PrimalItemArmor_FurGloves.PrimalItemArmor_FurGloves'"
    RIOT_LEGGINGS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Riot/PrimalItemArmor_RiotPants.PrimalItemArmor_RiotPants'"
    RIOT_CHESTPIECE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Riot/PrimalItemArmor_RiotShirt.PrimalItemArmor_RiotShirt'"
    RIOT_GAUNTLETS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Riot/PrimalItemArmor_RiotGloves.PrimalItemArmor_RiotGloves'"
    RIOT_BOOTS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Riot/PrimalItemArmor_RiotBoots.PrimalItemArmor_RiotBoots'"
    RIOT_HELMET = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Riot/PrimalItemArmor_RiotHelmet.PrimalItemArmor_RiotHelmet'"
    TEK_BOOTS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/TEK/PrimalItemArmor_TekBoots.PrimalItemArmor_TekBoots'"
    TEK_CHESTPIECE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/TEK/PrimalItemArmor_TekShirt.PrimalItemArmor_TekShirt'"
    TEK_GAUNTLETS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/TEK/PrimalItemArmor_TekGloves.PrimalItemArmor_TekGloves'"
    TEK_HELMET = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/TEK/PrimalItemArmor_TekHelmet.PrimalItemArmor_TekHelmet'"
    TEK_LEGGINGS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/TEK/PrimalItemArmor_TekPants.PrimalItemArmor_TekPants'"
    TEK_SHOULDER_CANNON = "Blueprint'/Game/Genesis/Items/Armor/PrimalItemArmor_ShoulderCannon.PrimalItemArmor_ShoulderCannon'"
    SCUBA_LEGGINGS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/SCUBA/PrimalItemArmor_ScubaPants.PrimalItemArmor_ScubaPants'"
    WOODEN_SHIELD = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Shields/PrimalItemArmor_WoodShield.PrimalItemArmor_WoodShield'"
    METAL_SHIELD = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Shields/PrimalItemArmor_MetalShield.PrimalItemArmor_MetalShield'"
    RIOT_SHIELD = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Shields/PrimalItemArmor_TransparentRiotShield.PrimalItemArmor_TransparentRiotShield'"
    TEK_SHIELD = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Shields/PrimalItemArmor_ShieldTek.PrimalItemArmor_ShieldTek'"
    GHILLIE_BOOTS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Ghillie/PrimalItemArmor_GhillieBoots.PrimalItemArmor_GhillieBoots'"
    GHILLIE_CHESTPIECE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Ghillie/PrimalItemArmor_GhillieShirt.PrimalItemArmor_GhillieShirt'"
    GHILLIE_GAUNTLETS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Ghillie/PrimalItemArmor_GhillieGloves.PrimalItemArmor_GhillieGloves'"
    GHILLIE_LEGGINGS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Ghillie/PrimalItemArmor_GhilliePants.PrimalItemArmor_GhilliePants'"
    GHILLIE_MASK = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/Ghillie/PrimalItemArmor_GhillieHelmet.PrimalItemArmor_GhillieHelmet'"
    GAS_MASK = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/SCUBA/PrimalItemArmor_GasMask.PrimalItemArmor_GasMask'"
    DESERT_CLOTH_BOOTS = "Blueprint'/Game/ScorchedEarth/Outfits/PrimalItemArmor_DesertClothBoots.PrimalItemArmor_DesertClothBoots'"
    DESERT_CLOTH_GLOVES = "Blueprint'/Game/ScorchedEarth/Outfits/PrimalItemArmor_DesertClothGloves.PrimalItemArmor_DesertClothGloves'"
    DESERT_GOGGLES_AND_HAT = "Blueprint'/Game/ScorchedEarth/Outfits/PrimalItemArmor_DesertClothGogglesHelmet.PrimalItemArmor_DesertClothGogglesHelmet'"
    DESERT_CLOTH_PANTS = "Blueprint'/Game/ScorchedEarth/Outfits/PrimalItemArmor_DesertClothPants.PrimalItemArmor_DesertClothPants'"
    DESERT_CLOTH_SHIRT = "Blueprint'/Game/ScorchedEarth/Outfits/PrimalItemArmor_DesertClothShirt.PrimalItemArmor_DesertClothShirt'"
    NIGHT_VISION_GOGGLES = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Armor/SCUBA/PrimalItemArmor_NightVisionGoggles.PrimalItemArmor_NightVisionGoggles'"
    HAZARD_SUIT_BOOTS = "Blueprint'/Game/Aberration/CoreBlueprints/Items/Armor/HazardSuit/PrimalItemArmor_HazardSuitBoots.PrimalItemArmor_HazardSuitBoots'"
    HAZARD_SUIT_GLOVES = "Blueprint'/Game/Aberration/CoreBlueprints/Items/Armor/HazardSuit/PrimalItemArmor_HazardSuitGloves.PrimalItemArmor_HazardSuitGloves'"
    HAZARD_SUIT_HAT = "Blueprint'/Game/Aberration/CoreBlueprints/Items/Armor/HazardSuit/PrimalItemArmor_HazardSuitHelmet.PrimalItemArmor_HazardSuitHelmet'"
    HAZARD_SUIT_PANTS = "Blueprint'/Game/Aberration/CoreBlueprints/Items/Armor/HazardSuit/PrimalItemArmor_HazardSuitPants.PrimalItemArmor_HazardSuitPants'"
    HAZARD_SUIT_SHIRT = "Blueprint'/Game/Aberration/CoreBlueprints/Items/Armor/HazardSuit/PrimalItemArmor_HazardSuitShirt.PrimalItemArmor_HazardSuitShirt'"
    MDSM = "Blueprint'/Game/Extinction/CoreBlueprints/Items/Saddle/PrimalItemArmor_MekBackpack_Shield.PrimalItemArmor_MekBackpack_Shield'"
    MOMI = "Blueprint'/Game/Extinction/CoreBlueprints/Items/Saddle/PrimalItemArmor_MekTransformer.PrimalItemArmor_MekTransformer'"
    MRLM = "Blueprint'/Game/Extinction/CoreBlueprints/Items/Saddle/PrimalItemArmor_MekBackpack_MissilePod.PrimalItemArmor_MekBackpack_MissilePod'"
    MSCM = "Blueprint'/Game/Extinction/CoreBlueprints/Items/Saddle/PrimalItemArmor_MekBackpack_SiegeCannon.PrimalItemArmor_MekBackpack_SiegeCannon'"



class Artifacts:
    SPECIMEN_IMPLANT = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_StartingNote.PrimalItem_StartingNote'"
    ARTIFACT_OF_THE_BRUTE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_12.PrimalItemArtifact_12'"
    ARTIFACT_OF_THE_CLEVER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_05.PrimalItemArtifact_05'"
    ARTIFACT_OF_THE_CUNNING = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_11.PrimalItemArtifact_11'"
    ARTIFACT_OF_THE_DEVIOUS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_04.PrimalItemArtifact_04'"
    ARTIFACT_OF_THE_DEVOURER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_07.PrimalItemArtifact_07'"
    ARTIFACT_OF_THE_HUNTER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_01.PrimalItemArtifact_01'"
    ARTIFACT_OF_THE_IMMUNE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_08.PrimalItemArtifact_08'"
    ARTIFACT_OF_THE_LOST = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifactAB_4.PrimalItemArtifactAB_4'"
    ARTIFACT_OF_THE_MASSIVE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_03.PrimalItemArtifact_03'"
    ARTIFACT_OF_THE_PACK = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_02.PrimalItemArtifact_02'"
    ARTIFACT_OF_THE_SKYLORD = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_06.PrimalItemArtifact_06'"
    ARTIFACT_OF_THE_STRONG = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_09.PrimalItemArtifact_09'"
    ARTIFACT_OF_THE_GATEKEEPER_SCORCHED_EARTH_ICONPNG = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifactSE_01.PrimalItemArtifactSE_01'"
    ARTIFACT_OF_THE_CRAG_SCORCHED_EARTH_ICONPNG = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifactSE_02.PrimalItemArtifactSE_02'"
    ARTIFACT_OF_THE_DESTROYER_SCORCHED_EARTH_ICONPNG = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifactSE_03.PrimalItemArtifactSE_03'"
    ARTIFACT_OF_THE_DEPTHS_ABERRATION_ICONPNG = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifactAB.PrimalItemArtifactAB'"
    ARTIFACT_OF_THE_SHADOWS_ABERRATION_ICONPNG = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifactAB_2.PrimalItemArtifactAB_2'"
    ARTIFACT_OF_THE_STALKER_ABERRATION_ICONPNG = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Artifacts/PrimalItemArtifactAB_3.PrimalItemArtifactAB_3'"
    ARTIFACT_OF_THE_CHAOS = "Blueprint'/Game/Extinction/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_Extinction_DesertKaiju.PrimalItemArtifact_Extinction_DesertKaiju'"
    ARTIFACT_OF_THE_GROWTH = "Blueprint'/Game/Extinction/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_Extinction_ForestKaiju.PrimalItemArtifact_Extinction_ForestKaiju'"
    ARTIFACT_OF_THE_VOID = "Blueprint'/Game/Extinction/CoreBlueprints/Items/Artifacts/PrimalItemArtifact_Extinction_IceKaiju.PrimalItemArtifact_Extinction_IceKaiju'"
    ALLOSAURUS_BRAIN = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Allo.PrimalItemResource_ApexDrop_Allo'"
    ALPHA_BASILISK_FANG = "Blueprint'/Game/Aberration/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Basilisk_Alpha.PrimalItemResource_ApexDrop_Basilisk_Alpha'"
    ALPHA_CARNOTAURUS_ARM = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_AlphaCarno.PrimalItemResource_ApexDrop_AlphaCarno'"
    ALPHA_CRYSTAL_TALON = "Blueprint'/Game/PrimalEarth/Dinos/CrystalWyvern/Mega/PrimalItemResource_ApexDrop_AlphaCrystalWyvern.PrimalItemResource_ApexDrop_AlphaCrystalWyvern'"
    ALPHA_KARKINOS_CLAW = "Blueprint'/Game/Aberration/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_CrabClaw.PrimalItemResource_ApexDrop_CrabClaw'"
    ALPHA_LEEDSICHTHYS_BLUBBER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_AlphaLeeds.PrimalItemResource_ApexDrop_AlphaLeeds'"
    ALPHA_MEGALODON_FIN = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_AlphaMegalodon.PrimalItemResource_ApexDrop_AlphaMegalodon'"
    ALPHA_MOSASAUR_TOOTH = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_AlphaMosasaur.PrimalItemResource_ApexDrop_AlphaMosasaur'"
    ALPHA_RAPTOR_CLAW = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_AlphaRaptor.PrimalItemResource_ApexDrop_AlphaRaptor'"
    ALPHA_REAPER_KING_BARB = "Blueprint'/Game/Aberration/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_ReaperBarb.PrimalItemResource_ApexDrop_ReaperBarb'"
    ALPHA_TUSOTEUTHIS_EYE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_AlphaTuso.PrimalItemResource_ApexDrop_AlphaTuso'"
    ALPHA_TYRANNOSAUR_TOOTH = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_AlphaRex.PrimalItemResource_ApexDrop_AlphaRex'"
    ALPHA_X_TRICERATOPS_SKULL = "Blueprint'/Game/Genesis/Missions/Retrieve/RetrieveItems/TrikeSkull/PrimalItemSkin_TrikeSkullHelmet_Retrieve.PrimalItemSkin_TrikeSkullHelmet_Retrieve'"
    ARGENTAVIS_TALON = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Argentavis.PrimalItemResource_ApexDrop_Argentavis'"
    BASILISK_SCALE = "Blueprint'/Game/Aberration/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Basilisk.PrimalItemResource_ApexDrop_Basilisk'"
    BASILOSAURUS_BLUBBER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Basilo.PrimalItemResource_ApexDrop_Basilo'"
    CORRUPT_HEART = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_RareDrop_CorruptHeart.PrimalItemResource_RareDrop_CorruptHeart'"
    CRYSTAL_TALON = "Blueprint'/Game/PrimalEarth/Dinos/CrystalWyvern/PrimalItemResource_ApexDrop_CrystalWyvern.PrimalItemResource_ApexDrop_CrystalWyvern'"
    FIRE_TALON = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_FireWyvern.PrimalItemResource_ApexDrop_FireWyvern'"
    GASBAGS_BLADDER = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_GasBag.PrimalItemResource_ApexDrop_GasBag'"
    GIGANOTOSAURUS_HEART = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Giga.PrimalItemResource_ApexDrop_Giga'"
    GOLDEN_STRIPED_MEGALODON_TOOTH = "Blueprint'/Game/Genesis/Missions/Retrieve/RetrieveItems/MegTeeth/PrimalItemResource_RetrieveMegTooth.PrimalItemResource_RetrieveMegTooth'"
    LIGHTNING_TALON = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_LightningWyvern.PrimalItemResource_ApexDrop_LightningWyvern'"
    MEGALANIA_TOXIN = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Megalania.PrimalItemResource_ApexDrop_Megalania'"
    MEGALODON_TOOTH = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Megalodon.PrimalItemResource_ApexDrop_Megalodon'"
    POISON_TALON = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_PoisonWyvern.PrimalItemResource_ApexDrop_PoisonWyvern'"
    REAPER_KING_PHEROMONE_GLAND = "Blueprint'/Game/Genesis/Missions/Retrieve/RetrieveItems/ReaperGland/PrimalItemResource_RetrieveReaperGland.PrimalItemResource_RetrieveReaperGland'"
    REAPER_PHEROMONE_GLAND = "Blueprint'/Game/Aberration/CoreBlueprints/Resources/PrimalItemResource_XenomorphPheromoneGland.PrimalItemResource_XenomorphPheromoneGland'"
    ROCK_DRAKE_FEATHER = "Blueprint'/Game/Aberration/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_RockDrake.PrimalItemResource_ApexDrop_RockDrake'"
    SARCOSUCHUS_SKIN = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Sarco.PrimalItemResource_ApexDrop_Sarco'"
    SAUROPOD_VERTEBRA = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Sauro.PrimalItemResource_ApexDrop_Sauro'"
    SPINOSAURUS_SAIL = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Spino.PrimalItemResource_ApexDrop_Spino'"
    THERIZINO_CLAWS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Theriz.PrimalItemResource_ApexDrop_Theriz'"
    THYLACOLEO_HOOK_CLAW = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Thylaco.PrimalItemResource_ApexDrop_Thylaco'"
    TITANOBOA_VENOM = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Boa.PrimalItemResource_ApexDrop_Boa'"
    TUSOTEUTHIS_TENTACLE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Tuso.PrimalItemResource_ApexDrop_Tuso'"
    TYRANNOSAURUS_ARM = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Rex.PrimalItemResource_ApexDrop_Rex'"
    YUTYRANNUS_LUNGS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ApexDrop_Yuty.PrimalItemResource_ApexDrop_Yuty'"



class Trophies:
    ALPHA_DEATHWORM_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_AlphaWorm.PrimalItemTrophy_AlphaWorm'"
    ALPHA_REX_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_AlphaRex.PrimalItemTrophy_AlphaRex'"
    ALPHA_WYVERN_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_AlphaWyvern.PrimalItemTrophy_AlphaWyvern'"
    BROODMOTHER_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Broodmother.PrimalItemTrophy_Broodmother'"
    ALPHA_BROODMOTHER_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Broodmother_Alpha.PrimalItemTrophy_Broodmother_Alpha'"
    BETA_BROODMOTHER_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Broodmother_Beta.PrimalItemTrophy_Broodmother_Beta'"
    GAMMA_BROODMOTHER_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Broodmother_Gamma.PrimalItemTrophy_Broodmother_Gamma'"
    MEGAPITHECUS_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Gorilla.PrimalItemTrophy_Gorilla'"
    ALPHA_MEGAPITHECUS_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Gorilla_Alpha.PrimalItemTrophy_Gorilla_Alpha'"
    BETA_MEGAPITHECUS_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Gorilla_Beta.PrimalItemTrophy_Gorilla_Beta'"
    GAMMA_MEGAPITHECUS_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Gorilla_Gamma.PrimalItemTrophy_Gorilla_Gamma'"
    DRAGON_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Dragon.PrimalItemTrophy_Dragon'"
    ALPHA_DRAGON_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Dragon_Alpha.PrimalItemTrophy_Dragon_Alpha'"
    BETA_DRAGON_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Dragon_Beta.PrimalItemTrophy_Dragon_Beta'"
    GAMMA_DRAGON_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Dragon_Gamma.PrimalItemTrophy_Dragon_Gamma'"
    MANTICORE_TROPHY = "Blueprint'/Game/ScorchedEarth/Structures/TrophyHeads/Manticore_Trophy/PrimalItemTrophy_Manticore.PrimalItemTrophy_Manticore'"
    ALPHA_MANTICORE_TROPHY = "Blueprint'/Game/ScorchedEarth/Structures/TrophyHeads/Manticore_Trophy/PrimalItemTrophy_Manticore_Alpha.PrimalItemTrophy_Manticore_Alpha'"
    BETA_MANTICORE_TROPHY = "Blueprint'/Game/ScorchedEarth/Structures/TrophyHeads/Manticore_Trophy/PrimalItemTrophy_Manticore_Beta.PrimalItemTrophy_Manticore_Beta'"
    GAMMA_MANTICORE_TROPHY = "Blueprint'/Game/ScorchedEarth/Structures/TrophyHeads/Manticore_Trophy/PrimalItemTrophy_Manticore_Gamma.PrimalItemTrophy_Manticore_Gamma'"
    ROCKWELL_TROPHY_ALPHA = "Blueprint'/Game/Aberration/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Rockwell_Beta_Alpha.PrimalItemTrophy_Rockwell_Beta_Alpha'"
    ROCKWELL_TROPHY_BETA = "Blueprint'/Game/Aberration/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Rockwell_Beta.PrimalItemTrophy_Rockwell_Beta'"
    ROCKWELL_TROPHY_GAMMA = "Blueprint'/Game/Aberration/CoreBlueprints/Items/Trophies/PrimalItemTrophy_Rockwell.PrimalItemTrophy_Rockwell'"
    DESERT_TITAN_TROPHY = "Blueprint'/Game/Extinction/CoreBlueprints/Trophies/PrimalItemTrophy_Kaiju_Desert.PrimalItemTrophy_Kaiju_Desert'"
    FOREST_TITAN_TROPHY = "Blueprint'/Game/Extinction/CoreBlueprints/Trophies/PrimalItemTrophy_Kaiju_Forest.PrimalItemTrophy_Kaiju_Forest'"
    ICE_TITAN_TROPHY = "Blueprint'/Game/Extinction/CoreBlueprints/Trophies/PrimalItemTrophy_Kaiju_Ice.PrimalItemTrophy_Kaiju_Ice'"
    KING_TITAN_TROPHY_ALPHA = "Blueprint'/Game/Extinction/CoreBlueprints/Trophies/PrimalItemTrophy_Kaiju_King_Alpha.PrimalItemTrophy_Kaiju_King_Alpha'"
    KING_TITAN_TROPHY_BETA = "Blueprint'/Game/Extinction/CoreBlueprints/Trophies/PrimalItemTrophy_Kaiju_King_Beta.PrimalItemTrophy_Kaiju_King_Beta'"
    KING_TITAN_TROPHY_GAMMA = "Blueprint'/Game/Extinction/CoreBlueprints/Trophies/PrimalItemTrophy_Kaiju_King_Gamma.PrimalItemTrophy_Kaiju_King_Gamma'"
    MOEDER_TROPHY = "Blueprint'/Game/Genesis/CoreBlueprints/Items/Trophies/PrimalItemTrophy_EelBoss.PrimalItemTrophy_EelBoss'"
    GAMMA_MOEDER_TROPHY = "Blueprint'/Game/Genesis/CoreBlueprints/Items/Trophies/PrimalItemTrophy_EelBoss_Gamma.PrimalItemTrophy_EelBoss_Gamma'"
    BETA_MOEDER_TROPHY = "Blueprint'/Game/Genesis/CoreBlueprints/Items/Trophies/PrimalItemTrophy_EelBoss_Beta.PrimalItemTrophy_EelBoss_Beta'"
    ALPHA_MOEDER_TROPHY = "Blueprint'/Game/Genesis/CoreBlueprints/Items/Trophies/PrimalItemTrophy_EelBoss_Alpha.PrimalItemTrophy_EelBoss_Alpha'"
    MASTER_CONTROLLER_TROPHY = "Blueprint'/Game/Genesis/CoreBlueprints/Items/Trophies/PrimalItemTrophy_MasterController.PrimalItemTrophy_MasterController'"
    CRYSTAL_WYVERN_QUEEN_TROPHY = "Blueprint'/Game/Mods/CrystalIsles/Assets/Dinos/CIBoss/Trophy/PrimalItemTrophy_CIBoss.PrimalItemTrophy_CIBoss'"
    GAMMA_CRYSTAL_WYVERN_QUEEN_TROPHY = "Blueprint'/Game/Mods/CrystalIsles/Assets/Dinos/CIBoss/Trophy/PrimalItemTrophy_CIBoss_Gamma.PrimalItemTrophy_CIBoss_Gamma'"
    BETA_CRYSTAL_WYVERN_QUEEN_TROPHY = "Blueprint'/Game/Mods/CrystalIsles/Assets/Dinos/CIBoss/Trophy/PrimalItemTrophy_CIBoss_Beta.PrimalItemTrophy_CIBoss_Beta'"
    ALPHA_CRYSTAL_WYVERN_QUEEN_TROPHY = "Blueprint'/Game/Mods/CrystalIsles/Assets/Dinos/CIBoss/Trophy/PrimalItemTrophy_CIBoss_Alpha.PrimalItemTrophy_CIBoss_Alpha'"
    SURVIVORS_TROPHY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Trophies/PrimalItemTrophy_ARKLogo.PrimalItemTrophy_ARKLogo'"


from pathlib import Path
from dataclasses import dataclass
from typing import Union, List, Optional

import win32clipboard, win32con


@dataclass
class Clip:
    type: str
    value: Union[str, List[Path]]


def read_clipboard() -> Optional[Clip]:
    try:
        win32clipboard.OpenClipboard()
        if win32clipboard.IsClipboardFormatAvailable(win32con.CF_HDROP):
            data: tuple = win32clipboard.GetClipboardData(win32con.CF_HDROP)
            return Clip('files', [Path(f) for f in data])
        elif win32clipboard.IsClipboardFormatAvailable(win32con.CF_UNICODETEXT):
            data: str = win32clipboard.GetClipboardData(win32con.CF_UNICODETEXT)
            return Clip('text', data)
        elif win32clipboard.IsClipboardFormatAvailable(win32con.CF_TEXT):
            data: bytes = win32clipboard.GetClipboardData(win32con.CF_TEXT)
            return Clip('text', data.decode())
        elif win32clipboard.IsClipboardFormatAvailable(win32con.CF_BITMAP):
            # TODO: handle screenshots
            pass
        return None
    finally:
        win32clipboard.CloseClipboard()


if __name__ == '__main__':
    print(read_clipboard())


from abc import abstractmethod
from pathlib import Path
import configparser
from collections import defaultdict


class Configuration:
    def __init__(self, path):
        self.path = path and Path(path) or None
        self.data = defaultdict(lambda: None)

    @abstractmethod
    def __getitem__(self, item):
        pass

    @abstractmethod
    def __contains__(self, item):
        pass

    @abstractmethod
    def __delitem__(self, key):
        pass

    @abstractmethod
    def __setitem__(self, key, value):
        pass

    @abstractmethod
    def items(self):
        pass

    @abstractmethod
    def keys(self):
        pass

    def exists(self):
        return self.path is not None and self.path.exists()


class Ini(Configuration):
    def __init__(self, path: str or Path = None):
        super().__init__(path)
        if self.path and self.path.exists():
            config = configparser.ConfigParser()
            config.read(self.path)
            self.data = self.__to_dict(config)

    @classmethod
    def __to_dict(cls, data: dict, result=None):
        if result is None:
            result = {}
        for key, value in data.items():
            if isinstance(value, configparser.SectionProxy):
                result[key] = cls.__to_dict(dict(value))
            else:
                if isinstance(value, str):
                    value = value.lower()
                    if value == 'false':
                        value = False
                    elif value == 'true':
                        value = True
                    elif value.isnumeric():
                        value = int(value)
                result[key] = value
        return result

    @classmethod
    def __to_array(cls, data: list):
        result = []
        for item in data:
            result.append(item)
        return list(data)

    def __getitem__(self, item):
        return self.data[item]

    def __contains__(self, item):
        return item in self.data

    def items(self):
        return self.data.items()

    def keys(self):
        return self.data.keys()

    def values(self):
        return self.data.values()

    def __iter__(self):
        return iter(self.data)

    def __str__(self):
        return str(self.data)

    def __repr__(self):
        items = []
        for k, v in self.__dict__.items():
            if k and k[0] != "_":
                items.append(f"\033[34m{k}\033[90m=\033[0m{repr(v)}\033[0m")
        args = ', '.join(items)
        return f'<\033[96m{type(self).__name__}\033[0m({args})>\033[0m'



from arklibrary.blueprints import Spectator, PlayerManagement, CreativeMode, ServerManagement
from arklibrary.blueprints import ItemManagement, DinoManagement, Teleportation, Map, TribeManagement
from pathlib import Path
import json


class Admin:
    def __init__(self, driver: 'Driver' = None, password='password', player_id=None):
        # ctrl-x to clear console
        # ctrl-c to copy console
        self.__password = password
        self.__my_player_id = player_id
        self.driver = driver
        self.command_list = []
        self.cache = {'default': player_id}

    def execute(self, *commands):
        if len(self.command_list) > 0:
            self.driver.write_console(*self.command_list, *commands)
            self.command_list.clear()

    def enable_admin(self):
        self.enable_cheats(self.__password)
        self.gcm()
        self.execute()
        return self

    def enable_cheats(self, password):
        self.command_list.append(CreativeMode.ENABLE_CHEATS.format(password))
        return self

    def gcm(self):
        self.command_list.append(CreativeMode.GIVE_CREATIVE_MODE)
        return self

    def god_mode(self):
        self.command_list.append(PlayerManagement.GOD)
        return self

    def clear_tutorials(self):
        self.command_list.append(PlayerManagement.CLEAR_TUTORIALS)
        return self

    def leave_me_alone(self):
        self.command_list.append(PlayerManagement.LEAVE_ME_ALONE)
        return self

    def enemy_invisible(self):
        self.command_list.append(PlayerManagement.ENEMY_INVISIBLE)
        return self

    def enemy_ignores_me(self, condition=True):
        """ Enemy creatures ignore me """
        condition = "true" if condition else "false"
        self.command_list.append(PlayerManagement.ENEMY_INVISIBLE.format(condition))
        return self

    def ghost(self):
        """ Allows to go through walls """
        self.command_list.append(PlayerManagement.GHOST)
        self.enemy_ignores_me()
        return self

    def enable_spectator(self):
        """ Once you enter the spectator state your body will disapear and your inventory items
        will be erased as well. If you leave the game and rejoin it will carry on where you left
        off, thus using the command "stopspectating" is highly recommended. The latter will
        show the respawn map. """
        self.command_list.append(Spectator.ENABLE_SPECTATOR)
        return self

    def stop_spectating(self):
        """ Stops spectating """
        self.command_list.append(Spectator.STOP_SPECTATING)
        return self

    def message_server(self, message):
        """ Writes to chat as a server message to all players"""
        self.command_list.append(ServerManagement.SERVER_CHAT.format(message))
        return self

    def broadcast(self, message):
        """ Sends a message to the whole server using the banner """
        self.command_list.append(ServerManagement.BROADCAST.format(str(message)))
        return self

    def give_item_to_player(self, player_id, blueprint_path, quantity=1, quality=0, force_blueprint=False):
        """ Adds the specified item (or its blueprint) to the player's inventory in the specified quantity
            Ex. cheat GiveItemToPlayer 696509819 "Blueprint'/Game/PrimalEarth/CoreBlueprints/Weapons/PrimalItemAmmo_AdvancedRifleBullet.PrimalItemAmmo_AdvancedRifleBullet'" 200 1 0
        """
        force_blueprint = 1 if force_blueprint else 0
        self.command_list.append(ItemManagement.GIVE_ITEM_TO_PLAYER.format(
            player_id, blueprint_path, quantity, quality, force_blueprint))
        return self

    def give_items_to_player(self, player_id, blueprint_paths, quantity=1, quality=0, force_blueprint=False):
        """ Adds the specified item (or its blueprint) to the player's inventory in the specified quantity
            Ex. cheat GiveItemToPlayer 696509819 "Blueprint'/Game/PrimalEarth/CoreBlueprints/Weapons/PrimalItemAmmo_AdvancedRifleBullet.PrimalItemAmmo_AdvancedRifleBullet'" 200 1 0
            NOTE: Quantity above the stack limit isn't given
        """
        force_blueprint = 1 if force_blueprint else 0
        for path in blueprint_paths:
            self.command_list.append(ItemManagement.GIVE_ITEM_TO_PLAYER.format(
                player_id, path, quantity, quality, force_blueprint))
        return self

    def spawn_exact_dino(self, dino_blueprint_path="", saddle_blueprint_path="", saddle_quality=0,
                         base_level=150, extra_levels=0, base_stats="0,0,0,0,0,0,1,1",
                         added_stats="0,0,0,0,0,0,0,0", dino_name="", cloned=0, neutered=0,
                         tamed_on="", uploaded_from="", imprinter_name="", imprinter_player_id=0,
                         imprint_quality=0, colors="0,0,0,0,0,0", dino_id=0, exp=0,
                         spawn_distance=0, y_offset=0, z_offset=0):
        """
        DinoBlueprintPath:	    String,	Blueprint path
        SaddleBlueprintPath:	String,	Blueprint path
        To spawn the creature without saddle pass an empty string ("")

        SaddleQuality:	        float,	Quality of the equipped saddle
        BaseLevel:	       Integer[32],	Level of the spawned Dino.
        If the level is not the sum of BaseStats + 1, the creature stats may be broken.

        ExtraLevels:	   Integer[32],	Sum of the domesticated levels
        If the value is not the sum of AddedStats, the creature stats may be broken.

        BaseStats:	            String,	Comma separated string with the base levels
        The order is health, stamina, oxygen, food, weight, melee damage, movement speed, crafting skill
        The set levels will only be visible after putting the creature in and out of a crypod.
        Syntax example: "0,0,0,0,0,0,0,0"

        AddedStats:	            String,	Comma separated string with the domesticated levels
        The order is health, stamina, oxygen, food, weight, melee damage, movement speed, crafting skill
        The set levels will only be visible after putting the creature in and out of a crypod.
        Syntax example: "0,0,0,0,0,0,0,0"

        DinoName:	            String
        Cloned:	            Integer[8]
        Neutered:	        Integer[8]
        TamedOn:	        String
        UploadedFrom:	        String
        ImprinterName:	        String,	If left empty, there will be no imprint
        ImprinterPlayerID:	Integer[32]
        ImprintQuality:	        float

        Colors:     	        String,	Comma separated string with the color ids
        Syntax example: "0,0,0,0,0,0"
        The colors will appear if the creature was put in and out of a cryopod.
        If an empty string ("") is passed, the colors will be randomly taken from the natural occuring colors of that species.

        DinoID:	           Integer[64],	If 0 is passed, the game will roll the id.
        Exp:	           Integer[64]
        spawnDistance:	        float
        YOffset:	            float
        ZOffset:	            float
        """
        format = DinoManagement.SPAWN_EXACT_DINO
        self.command_list.append(format.format(dino_blueprint_path, saddle_blueprint_path, saddle_quality, base_level,
                                   extra_levels, base_stats, added_stats, dino_name, cloned, neutered,
                                   tamed_on,
                                   uploaded_from, imprinter_name, imprinter_player_id, imprint_quality,
                                   colors,
                                   dino_id, exp, spawn_distance, y_offset, z_offset))
        return self

    def spawn_dino(self, blueprint_path, tamed=False, level=0):
        """
        Spawns a dino
        :param name_part:	String	significant part of a creature's Entity ID (without _C)
        :param tamed: 	    Boolean	true or 1: tamed, false or 0: wild
        :param level: 	    float	Give creature a specific level, set to 0 for random

        Ex. spawn_dino("dodo", True, 150)
        """
        tamed = "true" if tamed else "false"
        if 'bionics' in blueprint_path:
            name = blueprint_path.split("/")[-1].split('.')[0]
        else:
            name = blueprint_path.split("/")[4]
        self.command_list.append(DinoManagement.SDF.format(name, tamed, level))
        return self

    def remove_cryosickness(self):
        self.command_list.append(DinoManagement.CLEAR_CRYO_SICKNESS)
        return self

    def set_target_dino_color(self, color_region: int, color_id: int):
        """
        Change a dino's color
        :param color_region:    Integer[32]	Color Region
        :param color_id:        Integer[32]	Color ID.
        """
        self.command_list.append(DinoManagement.SET_TARGET_DINO_COLOR.format(color_region, color_id))
        return self

    def copy_coords(self):
        """ Copies your current coordinates and rotation to clipboard in the form x,y,z Yaw pitch """
        self.command_list.append(Map.COPY_COORDS_TO_CLIPBOARD)
        return self

    def teleport_exact(self, x, y, z, yaw, pitch):
        """ Pitch is looking down or up (-80.00 ... 87.00), and 0.00 is leveled"""
        """ Yaw is rotation left to right (-80.00 north... 0.00 east.. 80.00 south... 179.99 west)"""
        self.command_list.append(Teleportation.SPI.format(int(x), int(y), int(z), float(yaw), float(pitch)))
        return self

    def teleport_gps_coords(self, lattitude, longitude, altitude):
        """ Moves player to GPS coordinates """
        self.command_list.append(Teleportation.TP_COORDS.format(lattitude, longitude, altitude))
        return self

    def teleport_player_id_to_me(self, player_id):
        """
        Teleport player by player id to me
        :param id: int, player's id
        """
        self.command_list.append(Teleportation.TELEPORT_PLAYER_ID_TO_ME.format(player_id))
        return self

    def teleport_to_playerid(self, player_id):
        """ Teleport to another player by their id """

        self.command_list.append(Teleportation.TELEPORT_TO_PLAYER.format(player_id))
        return self

    def paste_teleport(self):
        """ After copying coordinates pastes and teleport here """
        copied = self.driver.app.get_from_clipboard()
        x, y, z, yaw, pitch = copied.split()
        self.command_list.append(Teleportation.SPI.format(int(x), int(y), int(z), float(yaw), float(pitch)))
        return self

    def spawn_beacon(self, blueprint_path):
        self.command_list.append(Map.SUMMON.format(blueprint_path))
        return self

    def teleport_green_obelisk(self):
        """ Teleport myself to green obelisk """

        self.command_list.append(Teleportation.TP.format("green"))
        return self

    def teleport_blue_obelisk(self):
        """ Teleport myself to blue obelisk """

        self.command_list.append(Teleportation.TP.format("blue"))
        return self

    def teleport_red_obelisk(self):
        """ Teleport myself to red obelisk """

        self.command_list.append(Teleportation.TP.format("red"))
        return self

    def teleport_xyz(self, x, y, z):
        """ Moves player to x,y,z game coordinates """
        self.command_list.append(Teleportation.SET_PLAYER_POS.format(x, y, z))
        return self

    def teleport_target_xyz(self, x, y, z):
        """ Moves player or dino to game coordinates """
        self.command_list.append(Teleportation.MOVE_TARGET_TO.format(x, y, z))
        return self

    def save_current_coordinates(self, map, name):
        """ Stores the coordinates into file with a name
        :param name: str, name for the coordinates
        """
        data = {"center": {}, "gen1": {}, "gen2": {}, "island": {},
                "scortched": {}, "ragnarok": {}, "extinction": {}, "aberration": {}}
        if map not in data:
            raise NotImplementedError("'{}' isn't part of the map list: {}".format(map, data.keys()))
        self.copy_coords()
        coords = [float(val) for val in self.driver.get_from_clipboard().split()]
        file_path = Path.cwd() / Path('../logs/saved_coordinates.json')
        if file_path.exists():
            with open(file_path, 'r') as r:
                data = json.load(r)

        cc = {"x": coords[0], "y": coords[1], "z": coords[2], "yaw": coords[3], "pitch": coords[4]}
        data[map][name] = cc
        with open(file_path, "w") as w:
            json.dump(data, w)

    def give_tek_engrams_to(self, player_id, engram=None):
        self.command_list.append(f"cheat GiveTekEngramsTo {player_id} tek")
        return self

    def hide_admin_icon(self, condition=True):
        """ Hides admin icon when writing into chat """
        condition = "true" if condition else "false"
        self.command_list.append("cheat SetAdminIcon {}".format(condition))
        return self

    def tranquilize_target(self):
        """ Tranquilize target in crosshairs """
        self.command_list.append("cheat SetMyTargetSleeping 1")
        return self

    def wake_up_target(self):
        """ Removes tranquilizing effects of target in crosshairs """
        self.command_list.append("cheat SetMyTargetSleeping 0")
        return self

    def add_hexagons(self, amount):
        """ Give myself a number of hexagons """
        self.command_list.append("cheat AddHexagons {}".format(amount))
        return self

    def tame(self):
        """ Tames the targeted dino in my crosshairs """
        self.command_list.append("cheat DoTame")
        return self


    def clear_player_inventory(self, player_id, inventory=True, slot_items=False, equipment=False):
        """ Clears player's inventory, equipped items, and/or slot items.
            Ex. cheat ClearPlayerInventory <playerID> <inventory?> <slotitems?> <equippeditems?>
                cheat ClearPlayerInventory 4213211323 1 1 1 1
        """
        # TODO: Need to fix numbers (e.g. clear_player_inventory(id, True, False, False) -> clear_player_inventory id 1 1 1
        # Should be returning 1 0 0 at the end instead of 1 1 1
        inventory = "1" if inventory else "0"
        slot_items = "1" if inventory else "0"
        equipment = "1" if inventory else "0"

        self.command_list.append(PlayerManagement.CLEAR_PLAYER_INVENTORY.format(player_id, inventory, slot_items, equipment))

    def rename_player(self, current_name, new_name):
        self.command_list.append(PlayerManagement.RENAME_PLAYER.format(current_name, new_name))
        return self

    def rename_tribe(self, current_name, new_name):
        self.command_list.append(TribeManagement.RENAME_TRIBE.format(current_name, new_name))
        return self

    def show_admin_manager(self):
        """ shows a list of all the players with their player id's """
        self.command_list.append(ServerManagement.SHOW_MY_ADMIN_MANAGER)
        return self

    def ban_player(self, player_id):
        """ Bans a player """
        self.command_list.append(ServerManagement.BAN_PLAYER.format(player_id))
        return self

    def unban_player(self, player_id):
        """ Unbans player """
        self.command_list.append(ServerManagement.UNBAN_PLAYER.format(player_id))
        return self

    def disable_spectator(self):
        self.command_list.append(Spectator.STOP_SPECTATING)
        return self

    def teleport(self, command):
        self.command_list.append(Teleportation.TP.format(command))
        return self

    def teleport_king_titan_terminal(self):
        """ Teleport me to King Titan Terminal in Extinction"""

        self.command_list.append(Teleportation.TP.format("king"))
        return self

    def teleport_desert_titan_terminal(self):
        """ Teleport me to Desert Titan Terminal in Extinction"""
        self.command_list.append(Teleportation.TP.format("red"))
        return self

    def teleport_forest_titan_terminal(self):
        """ Teleport me to Forest Titan Terminal in Extinction"""
        self.command_list.append(Teleportation.TP.format("green"))
        return self

    def teleport_ice_titan_terminal(self):
        """ Teleport me to Ice Titan Terminal in Extinction"""
        self.command_list.append(Teleportation.TP.format("blue"))
        return self

    def teleport_to_player_name(self, name):
        """ Teleport to another player by their name """
        self.command_list.append(Teleportation.TELEPORT_TO_PLAYER_NAME.format(name))
        return self

    def teleport_player_name_to_me(self, name):
        """
        Teleport player by name to me.
        :param name: str, player's ingame name
        """
        self.command_list.append(Teleportation.TELEPORT_PLAYER_NAME_TO_ME.format(name))
        return self

    def kill_target(self):
        """ Kills exactly whoever is in my crosshairs """
        self.command_list.append(DinoManagement.DESTROY_MY_TARGET)
        return self

    def destroy_tribes_dinos(self):
        """ Kills all dinos in my crosshairs in its tribe """
        self.command_list.append(DinoManagement.DESTROY_TRIBE_DINOS)
        return self

    def destroy_tribes_dinos_by_id(self, tribe_id):
        """ Kills all tribe's dinos """
        self.command_list.append(TribeManagement.DESTROY_TRIBE_ID.format(tribe_id))
        return self

    def destroy_players_by_tribeid(self, tribe_id):
        """ Destroys all players in a tribe by tribe id """
        self.command_list.append(TribeManagement.DESTROY_TRIBE_ID_PLAYERS.format(tribe_id))
        return self

    def destroy_tribe_by_id(self, tribe_id):
        """ Destroy a tribe by their id """
        self.command_list.append(TribeManagement.DESTROY_TRIBE_ID.format(tribe_id))
        return self

    def destroy_structures_by_tribeid(self, tribe_id):
        """ Destroys all structures in a tribe by tribe id """
        self.command_list.append(TribeManagement.DESTROY_TRIBE_ID_STRUCTURES.format(tribe_id))
        return self

    def destroy_tribe_structures(self):
        """ destroys target tribe's structures """
        self.command_list.append(TribeManagement.DESTROY_TRIBE_STRUCTURES)
        return self

    def destroy_wild_dinos(self):
        """ Destroys all untamed dinos to help with respawns """
        self.command_list.append(DinoManagement.DESTROY_WILD_DINOS)
        return self

    def force_player_to_join_target_tribe(self, player_id):
        """ Force a player to join a tribe i'm targeting with my crosshairs"""
        self.command_list.append(TribeManagement.FORCE_PLAYER_TO_JOIN_TARGET_TRIBE.format(player_id))
        return self

    def force_myself_into_target_tribe(self):
        """ Force myself into a tribe I'm targeting with my crosshairs """
        self.command_list.append(TribeManagement.FORCE_JOIN_TRIBE)
        return self

    def force_myself_tribe_admin(self):
        """ Force myself as tribe admin """
        self.command_list.append(TribeManagement.FORCE_TRIBE_ADMIN)
        return self

    def force_myself_tribe_owner(self):
        """ Force myself as tribe owner """
        self.command_list.append(TribeManagement.FORCE_TRIBE_FOUNDER)
        return self

    def give_experience_to_player(self, player_id, amount, from_tribe_share=False, prevent_sharing_with_tribe=True):
        """ Give a player experience points """
        from_tribe_share = 1 if from_tribe_share else 0
        self.command_list.append(
            PlayerManagement.GIVE_EXP_TO_PLAYER.format(player_id, amount, from_tribe_share,
                                                       prevent_sharing_with_tribe))
        return self

    def add_experience_on_mounted_dino(self, amount, from_tribe_share=False, prevent_sharing_with_tribe=True):
        """ Give experience to a dino you're mounted on """
        from_tribe_share = 1 if from_tribe_share else 0
        prevent_sharing_with_tribe = 1 if prevent_sharing_with_tribe else 0
        self.command_list.append(
            PlayerManagement.ADD_EXPERIENCE.format(amount, from_tribe_share, prevent_sharing_with_tribe))
        return self

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.execute()

    def restart_application(self):
        self.driver.app.close()
        return Admin(password=self.__password)






import inspect

COLORS = {
    'red': (188, 63, 60),
    'yellow': (187, 181, 41),
    'orange': (204, 120, 50),
    'blue': (40, 123, 22),
    'green': (106, 135, 89),
    'cyan': (104, 151, 187),
    'purple': (152, 118, 170),
    'brown': (138, 101, 59),
    'error': (255, 107, 104),
    'gray': (85, 85, 85),
    'white': (187, 187, 187)
}


def _foreground(r=None, g=None, b=None):
    if r or g or b:
        r = r or 0
        g = g or 0
        b = b or 0
        return f"38;2;{r};{g};{b}"
    return None


def _background(r=None, g=None, b=None):
    if r or g or b:
        r = r or 0
        g = g or 0
        b = b or 0
        return f"48;2;{r};{g};{b}"
    return None


def _decor(decoration):
    if not decoration:
        return None
    elif 'bold' in decoration:
        return '1'
    elif 'italicize' in decoration:
        return '3'
    elif 'underline' in decoration:
        return '4'
    else:
        return None


def colorize(message, color=None, bg=None, decoration=None):
    color = color and _foreground(*COLORS[color]) or None
    bg = bg and _background(*COLORS[bg]) or None
    decoration = _decor(decoration)
    valid_escapes = [e for e in [decoration, color, bg] if e]
    if valid_escapes:
        escapes = ';'.join(valid_escapes)
        return f'\033[{escapes}m{message}\033[0m'
    return str(message)


def error(*messages, title='ERR'):
    title = colorize("{:^10}".format(title), color='white', bg='red', decoration='bold')
    message_str = ', '.join([str(m) for m in messages])
    message = colorize(message_str, color='error')
    print(f"{title}\t{message}")
    return message_str


def success(*messages):
    title = colorize("{:^10}".format("DONE"), color='white', bg='green', decoration='bold')
    message_str = ', '.join([str(m) for m in messages])
    message = colorize(message_str, color='green')
    print(f"{title}\t{message}")
    return message_str


def log(message):
    title = colorize("{:^10}".format("LOG"), color='white', bg='gray')
    print(f"{title}\t{message}")
    return message


def condition(cond=None, expr=None, result=False):
    title = colorize("{:^10}".format("COND"), color='white', bg='purple')
    result = colorize(result, color='orange')
    expr = colorize(expr, color='cyan')
    arrow = ": -> "
    if cond:
        cond = colorize(cond, color='purple', decoration='bold')
        l_paren = colorize(" (", color='purple', decoration='bold')
        r_paren = colorize(")", color='purple', decoration='bold')
        print(f"{title}\t{cond}{l_paren}{expr}{r_paren}{arrow}{result}")
        return f"{cond}{l_paren}{expr}{r_paren}{arrow}{result}"
    print(f"{title}\t{expr}{arrow}{result}")
    return f"{expr}{arrow}{result}"


def data(name=None, value=None, tabs=1):
    title = colorize("{:^10}".format(f"{name or 'DATA'}"), color='white', bg='cyan', decoration='bold')
    print(f"{title}\n" + "\t" * (tabs - 1) + f"{_colored_value(value, tabs=tabs)}")
    return str(value)


def _can_be_dict(value):
    from_class = inspect.isclass(value) and hasattr(value, 'keys') and hasattr(value, '__getitem__')
    from_dict = type(value) == dict
    return from_dict or from_class


def _colored_dict(value, tabs=1):
    result = "{\n"
    for k, v in value.items():
        result += "\t" * tabs + colorize(k, color='cyan') + ": "
        if _can_be_dict(v):
            result += _colored_dict(dict(v), tabs=tabs + 1)
        else:
            result += "" * tabs + f"{v},\n"
    return result + "\t" * (tabs - 1) + "}\n"


def _colored_value(value, tabs=1):
    if _can_be_dict(value):
        return _colored_dict(value, tabs=tabs)
    return f"{value}"


if __name__ == "__main__":
    mydict = {'one': 1, 'two': {'three': 3, 'four': 4, }}
    print(error("ERROR", 'error with syntax', mydict))
    print(success("Data is valid", mydict))
    print(condition("if", 'type(mydict) == dict', type(mydict) == dict))
    print(log("checking data"))
    print(data("mydict", mydict))



import threading
import ctypes
import win32api, win32gui

class Clipboard:
    def _create_window(self) -> int:
        """
        Create a window for listening to messages
        :return: window hwnd
        """
        wc = win32gui.WNDCLASS()
        wc.lpfnWndProc = self._process_message
        wc.lpszClassName = self.__class__.__name__
        wc.hInstance = win32api.GetModuleHandle(None)
        class_atom = win32gui.RegisterClass(wc)
        return win32gui.CreateWindow(class_atom, self.__class__.__name__, 0, 0, 0, 0, 0, 0, 0, wc.hInstance, None)

    def _process_message(self, hwnd: int, msg: int, wparam: int, lparam: int):
        WM_CLIPBOARDUPDATE = 0x031D
        if msg == WM_CLIPBOARDUPDATE:
            print('clipboard updated!')
        return 0

    def listen(self):
        def runner():
            hwnd = self._create_window()
            ctypes.windll.user32.AddClipboardFormatListener(hwnd)
            win32gui.PumpMessages()

        th = threading.Thread(target=runner, daemon=True)
        th.start()
        while th.is_alive():
            th.join(0.25)


import ctypes
import threading
from dataclasses import dataclass
from pathlib import Path
from typing import Callable, Union, List, Optional

import win32api, win32clipboard, win32con, win32gui


class Clipboard:
    @dataclass
    class Clip:
        type: str
        value: Union[str, List[Path]]

    def __init__(
            self,
            trigger_at_start: bool = False,
            on_text: Callable[[str], None] = None,
            on_update: Callable[[Clip], None] = None,
            on_files: Callable[[str], None] = None,
    ):
        self._trigger_at_start = trigger_at_start
        self._on_update = on_update
        self._on_files = on_files
        self._on_text = on_text

    def _create_window(self) -> int:
        """
        Create a window for listening to messages
        :return: window hwnd
        """
        wc = win32gui.WNDCLASS()
        wc.lpfnWndProc = self._process_message
        wc.lpszClassName = self.__class__.__name__
        wc.hInstance = win32api.GetModuleHandle(None)
        class_atom = win32gui.RegisterClass(wc)
        return win32gui.CreateWindow(class_atom, self.__class__.__name__, 0, 0, 0, 0, 0, 0, 0, wc.hInstance, None)

    def _process_message(self, hwnd: int, msg: int, wparam: int, lparam: int):
        WM_CLIPBOARDUPDATE = 0x031D
        if msg == WM_CLIPBOARDUPDATE:
            self._process_clip()
        return 0

    def _process_clip(self):
        clip = self.read_clipboard()
        if not clip:
            return

        if self._on_update:
            self._on_update(clip)
        if clip.type == 'text' and self._on_text:
            self._on_text(clip.value)
        elif clip.type == 'files' and self._on_text:
            self._on_files(clip.value)

    @staticmethod
    def read_clipboard() -> Optional[Clip]:
        try:
            win32clipboard.OpenClipboard()

            def get_formatted(fmt):
                if win32clipboard.IsClipboardFormatAvailable(fmt):
                    return win32clipboard.GetClipboardData(fmt)
                return None

            if files := get_formatted(win32con.CF_HDROP):
                return Clipboard.Clip('files', [Path(f) for f in files])
            elif text := get_formatted(win32con.CF_UNICODETEXT):
                return Clipboard.Clip('text', text)
            elif text_bytes := get_formatted(win32con.CF_TEXT):
                return Clipboard.Clip('text', text_bytes.decode())
            elif bitmap_handle := get_formatted(win32con.CF_BITMAP):
                # TODO: handle screenshots
                pass

            return None
        finally:
            win32clipboard.CloseClipboard()

    def listen(self):
        if self._trigger_at_start:
            self._process_clip()

        def runner():
            hwnd = self._create_window()
            ctypes.windll.user32.AddClipboardFormatListener(hwnd)
            win32gui.PumpMessages()

        th = threading.Thread(target=runner, daemon=True)
        th.start()
        while th.is_alive():
            th.join(0.25)


if __name__ == '__main__':
    clipboard = Clipboard(on_update=print, trigger_at_start=True)
    clipboard.listen()


if __name__ == '__main__':
    clipboard = Clipboard()
    clipboard.listen()





def pathify(text: str = None, class_name: str = None, text_content=None, **kwargs):
    paths = []
    if class_name:
        paths.append(f"@class='{class_name}'")
    if text:
        paths.append(f"text()='{text}'")
    if text_content:
        paths.append(f"normalize-space(.)='{text_content}'")
    for key, value in kwargs.items():
        if '_contains' in key:
            length = len('_contains')
            attribute = key[:-length]
            if attribute == 'class_name':
                attribute = "class"
            if attribute == 'text':
                paths.append(f"contains(text(), '{value}')")
            elif attribute == 'text_content':
                paths.append(f"contains(normalize-space(.), '{value}')")
            else:
                paths.append(f"contains(@{attribute}, '{value}')")
        elif '_starts_with' in key:
            length = len('_starts_with')
            attribute = key[:-length]
            if attribute == 'class_name':
                attribute = "class"
            if attribute == 'text':
                paths.append(f"starts-with(text(), '{value}')")
            elif attribute == 'text_content':
                paths.append(f"starts-with(normalize-space(.), '{value}')")
            else:
                paths.append(f"starts-with(@{attribute}, '{value}')")
        elif '_ends_with' in key:
            length = len('_ends_with')
            attribute = key[:-length]
            if attribute == 'class_name':
                attribute = "class"
            if attribute == 'text':
                paths.append(f"ends-with(text(), '{value}')")
            elif attribute == 'text_content':
                paths.append(f"ends-with(normalize-space(.), '{value}')")
            else:
                paths.append(f"ends-with(@{attribute}, '{value}')")
        elif '_regex' in key:
            length = len('_regex')
            attribute = key[:-length]
            if attribute == 'class_name':
                attribute = "class"
            if attribute == "text":
                paths.append(f"matches(text(), '{value}')")
            elif attribute == 'text_content':
                paths.append(f"matches(normalize-space(.), '{value}')")
            else:
                paths.append(f"matches(@{attribute}, '{value}')")
        else:
            paths.append(f"@{key}='{value}'")
    if paths:
        return '[' + ' and '.join(paths) + ']'
    return ''


class Logger:
    LABEL_WIDTH = 11
    SPACING = 3

    def __init__(self):
        self.__messages = []
        self.__has_errors = False
        self.__is_testing = False

    @property
    def is_testing(self):
        return self.__is_testing

    @property
    def has_errors(self):
        return self.__has_errors

    def messages(self):
        return '\n'.join(self.__messages)

    def __enter__(self):
        self.__is_testing = True
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.has_errors:
            for message in self.__messages:
                print(message)

    def error(self, message: str, **kwargs):
        label = "ERROR".center(Logger.LABEL_WIDTH)
        spacing = " " * Logger.SPACING
        colored_label = f"\033[0;1;41m{label}\033[0m"
        output = f"{colored_label}{spacing}\033[31m{message}\033[0m"
        self.__messages.append(output)
        self.__has_errors = True
        if not self.is_testing:
            print(output)
            return output

    def success(self, message, **kwargs):
        label = "SUCCESS".center(Logger.LABEL_WIDTH)
        spacing = " " * Logger.SPACING
        colored_label = f"\033[0;1;42m{label}\033[0m"
        output = f"{colored_label}{spacing}\033[32m{message}\033[0m"
        self.__messages.append(output)
        if not self.is_testing:
            print(output)
            return output

    def warning(self, message, **kwargs):
        label = "WARNING".center(Logger.LABEL_WIDTH)
        spacing = " " * Logger.SPACING
        colored_label = f"\033[0;1;43m{label}\033[0m"
        output = f"{colored_label}{spacing}\033[33m{message}\033[0m"
        self.__messages.append(output)
        if not self.is_testing:
            print(output)
            return output

    def log(self, message, **kwargs):
        label = "LOG".center(Logger.LABEL_WIDTH)
        spacing = " " * Logger.SPACING
        colored_label = f"\033[0;1;47m{label}\033[0m"
        output = f"{colored_label}{spacing}\033[37m{message}\033[0m"
        self.__messages.append(output)
        if not self.is_testing:
            print(output)
            return output

    def variables(self, **values):
        label = "VARIABLE".center(Logger.LABEL_WIDTH)
        spacing = " " * Logger.SPACING
        colored_label = f"\033[0;1;44m{label}\033[0m"
        for var_name, var_value in values.items():
            output = f"{colored_label}{spacing}\033[34m{var_name} = {repr(var_value)}\033[0m"
            self.__messages.append(output)
        if not self.is_testing:
            result = '\n'.join(self.__messages[-len(values):])
            print(result)
            return result

    def condition(self, statement, result, **kwargs):
        label = "CONDITION".center(Logger.LABEL_WIDTH)
        spacing = " " * Logger.SPACING
        colored_label = f"\033[0;1;45m{label}\033[0m"
        output = f"{colored_label}{spacing}\033[35m{statement} : {result}\033[0m"
        self.__messages.append(output)
        if not self.is_testing:
            print(output)
            return output


if __name__ == "__main__":
    log = Logger()
    log.error("an error occurred")
    log.warning("a warning occurred")
    log.success("something has passed")
    log.variables(value='string value')
    log.log('a simple log')
    log.condition('if a > 0', True)
    print("\n\n\n")

    print("-----------Should not have anything after here ----------")
    with Logger() as log:
        log.warning("a warning occurred")
        log.success("something has passed")
        log.variables(value='string value')
        log.log('a simple log')
        log.condition('if a > 0', True)
    print("-----------Should not have anything before here ----------\n\n\n\n")

    with Logger() as log:
        log.error("an error occurred")
        log.warning("a warning occurred")
        log.success("something has passed")
        log.variables(value='string value')
        log.log('a simple log')
        log.condition('if a > 0', True)




from bs4 import BeautifulSoup
from pathlib import Path
import requests
import json
from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.remote.webelement import WebElement
from time import sleep
import re
import difflib
import numpy as np
import cv2

creatures_folder = Path.cwd() / Path('xd/creatures')
items_folder = Path.cwd() / Path('xd/items')


def get_driver():
    d = webdriver.Chrome(
        executable_path=r"C:\Users\optimus\OneDrive\Developer\.id\.drivers\chrome\windows\94.0.4606.61\chromedriver.exe")
    return d


def iter_web_links(file: Path):
    data = []
    with open(file, 'r') as r:
        data = json.load(r)
    for item in data:
        yield item['name'], item['url']


def get_dino_img(driver: webdriver.Chrome, name: str):
    try:
        module = driver.find_element_by_xpath("//div[@class='info-arkitex info-module']")
        images = module.find_elements_by_tag_name("img")
    except:
        return []
    match = r'.*\.png|.*\.jpg|.*\.svg'
    links = []
    for img in images:
        try:
            alt = img.get_property('alt').lower()
            fromatted_alt = alt.replace("dossier ", "").replace("image", "").replace("img", "").replace(".svg", "")\
                .replace(".png", "").replace(".jpg", "").replace(".jpeg", "").replace("aberrant", "").replace("r-", "")\
                .replace("corrupted", "").replace("(alpha)", "").replace("(beta)", "")\
                .replace("(gamma)", "").replace("tek", "").replace("malfunctioned", "").replace("x-", "").strip()
            formatted_alt = "rock elemental" if "golem" in formatted_alt else formatted_alt
            formatted_name = name.lower().replace("aberrant", "").replace("r-", "").replace("corrupted", "")\
                .replace("brute", "").replace("tamed", "").replace("(alpha)", "").replace("(beta)", "")\
                .replace("(gamma)", "").replace("tek", "").replace("  ", " ").strip()
            seq = difflib.SequenceMatcher(None, formatted_name, fromatted_alt)
            if alt is not None:
                # links.append(img.get_property('src'))
                url = re.findall(match, img.get_property('src'))
                if len(url) > 0:
                    links += [(seq.ratio(), url[0])]
        except Exception as e:
            print(e)
    chosen = list(np.unique([l[1] for l in links if l[0] > 0.5]))
    if len(chosen) == 0:
        print('\033[34m' + f"[{name}]: {chosen}" + '\033[0m')
        text = input(f"Would you like to manually enter image url for {name}? ")
        if text is None or text.strip() == '':
            return chosen
        else:
            url = re.findall(match, text)
            chosen.append(url[0])
        text = input(f"Would you like to manually enter image url for {name}? ")
        if text is None or text.strip() == '':
            return chosen
        else:
            url = re.findall(match, text)
            chosen.append(url[0])
    if len(chosen) <= 0:
        print('\033[31m' + f"[{name}] Error: Didn't find images." + '\033[0m')
        raise Exception("Unable to find or choose a suitable an image url.")
    elif len(chosen) == 1:
        print('\033[34m' + f"[{name}]: {chosen}" + '\033[0m')
        text = input(f"Would you like to manually enter image url for {name}? ")
        if text is None or text.strip() == '':
            return chosen
        else:
            url = re.findall(match, text)
            chosen.append(url[0])
    img_weights = []
    for image in chosen:
        content = requests.get(image).content
        cv = cv2.imdecode(np.frombuffer(content, np.uint8), cv2.IMREAD_COLOR)
        area = cv.shape[0] * cv.shape[1]
        img_weights.append((area, image))
    img_weights.sort()
    sorted_chosen = [l[1] for l in img_weights]
    print('\033[92m' + f"[{name}]: {sorted_chosen}" + '\033[0m')
    return sorted_chosen


def get_item_img(driver: webdriver.Chrome, name: str):
    try:
        module = driver.find_element_by_xpath("//div[@class='info-arkitex info-module']")
        images = module.find_elements_by_tag_name("img")
    except:
        return []
    match = r'.*\.png|.*\.jpg|.*\.svg'
    links = []
    for img in images:
        try:
            alt = img.get_property('alt').lower()
            fromatted_alt = alt.replace("dossier ", "").replace("image", "").replace("img", "").replace(".svg", "")\
                .replace(".png", "").replace(".jpg", "").replace(".jpeg", "").replace("", "").replace("(", "")\
                .replace(")", "").replace("scortched earth", "").replace("extinction", "").replace("genesis", "")\
                .replace("%29", "").replace("%28", "").replace("aberration", "").strip()
            formatted_name = name.lower().replace("  ", " ").strip()
            seq = difflib.SequenceMatcher(None, formatted_name, fromatted_alt)
            if alt is not None:
                # links.append(img.get_property('src'))
                url = re.findall(match, img.get_property('src'))
                if len(url) > 0:
                    links += [(seq.ratio(), url[0])]
        except Exception as e:
            print(e)
    chosen = list(np.unique([l[1] for l in links if l[0] > 0.5]))
    if len(chosen) == 0:
        print('\033[34m' + f"[{name}]: {chosen}" + '\033[0m')
        text = input(f"Would you like to manually enter image url for {name}? ")
        if text is None or text.strip() == '':
            return None
        else:
            url = re.findall(match, text)
            chosen.append(url[0])
    if len(chosen) <= 0:
        print('\033[31m' + f"[{name}] Error: Didn't find images." + '\033[0m')
        raise Exception("Unable to find or choose a suitable an image url.")
    elif len(chosen) > 1:
        print('\033[34m' + f"[{name}]: {chosen}" + '\033[0m')
        text = input(f"Would you like to manually enter the ONLY image url for {name}? ")
        if text is None or text.strip() == '':
            return list(chosen)
        else:
            url = re.findall(match, text)
            print('\033[92m' + f"[{name}]: {url[0]}" + '\033[0m')
            return str(url[0])
    print('\033[92m' + f"[{name}]: {str(chosen[0])}" + '\033[0m')
    return str(chosen[0])


def save_image(content: bytes, url: str, name: str, map_file: Path, path: Path):
    relative = path / Path('images')
    root_dir = path / Path('images')
    if not root_dir.exists():
        root_dir.mkdir()
    json_path = path / Path('images/paths.json')
    if not json_path.exists():
        with open(json_path, 'w') as w:
            json.dump({}, w)
    extension = Path(url).suffix
    folder = relative / Path(map_file.stem)
    with open(json_path, 'r') as r:
        data = json.load(r)

    if name not in data:
        data[name] = []

    formatted = name.lower().replace("-", " ").replace(".", " ").replace("'", "")\
        .replace("`", "").replace(" ", "_").replace("\"", "")
    stem = f"{formatted}_{len(data[name])}"


    file = folder / Path(stem + extension)
    data[name].append(str(file.relative_to(relative)))

    with open(json_path, 'w') as w:
        json.dump(data, w)

    if not folder.is_dir():
        folder.mkdir()

    with open(file, 'wb') as w:
        w.write(content)

    print('\033[92m' + f"[SAVED] {name}: {str(file.relative_to(relative))}" + '\033[0m')


def get_description(driver: webdriver.Chrome):
    try:
        e = driver.find_element_by_xpath("//div[@class='dossier-background dossier-text']")
        print('\033[92m' + f"[DESC]: {e.text}" + '\033[0m')
        return e.text.replace("\n", "").replace("  ", "")
    except Exception as e:
        print(e)


def get_color_regions(driver: webdriver.Chrome):
    match = r'.*\.png|.*\.jpg|.*\.svg'

    try:
        data = {0: None, 1: None, 2: None, 3: None, 4: None, 5: None}
        container = driver.find_element_by_class_name("paintregion-container")
        regions = container.find_elements_by_xpath("//div[@class='paintregion']")[:6]
        for i, reg in enumerate(regions):
            if 'not used' not in reg.text.lower():
                data[i] = {}
                data[i]['text'] = reg.text.replace("\n", " ")
                src = reg.find_element_by_tag_name("img").get_property('src')
                imgs = re.findall(match, src)
                if len(imgs) > 0:
                    data[i]['img'] = imgs[0]
                data[i]['color_ids'] = {}
                data[i]['rgb'] = []
                color_squares = reg.find_elements_by_class_name("color-square")
                for square in color_squares:
                    text = square.get_property('title')
                    color, code = text.replace(")", "").split("\xa0(")
                    data[i]['color_ids'][color] = int(code)
                    style = square.get_attribute('style')
                    html_color = re.findall(r'[0-9]+, [0-9]+, [0-9]+', style)[0]
                    data[i]['rgb'].append([int(v) for v in html_color.split(", ")])
        return data
    except Exception as e:
        print('\033[31m' + "[COLOR] Unable to locate color regions for this creature." + '\033[0m')
        print(e)


def get_short_description(driver: webdriver.Chrome):
    try:
        quote = driver.find_elements_by_xpath("//td[@class='quote-left']//parent::tr")[0].text
        return quote.replace("\n", " ").replace("“", "").replace("„", "").strip()
    except Exception as e:
        print('\033[31m' + "[DESC] Unable to locate a description for this creature." + '\033[0m')
        raise e


def get_by_name_from_map(name:str, map: Path):
    with open(map, 'r') as r:
        dinos = json.load(r)
    for dino in dinos:
        if name in dino:
            return dino


def get_creature_data():
    driver = get_driver()
    try:
        sleep(2)
        creatures_folder = Path.cwd() / Path('creatures')
        for file in creatures_folder.iterdir():
            data = {}
            with open(file, 'r') as r:
                data = json.load(r)
            iterlinks = iter_web_links(file)
            for name, link in iterlinks:
                driver.get(link)
                sleep(0.5)
                try:
                    if "brute" not in name.lower():
                        img = get_dino_img(driver, name)
                        desc = get_description(driver)
                        color = get_color_regions(driver)
                        for url in img:
                            content = requests.get(url).content
                            save_image(content, url, name, file, creatures_folder)
                    else:
                        img = None
                        desc = None
                        color = None
                    for i in range(len(data)):
                        if name in data[i]['name']:
                            data[i]['images'] = img
                            data[i]['description'] = desc
                            data[i]['color_regions'] = color
                except Exception as e:
                    print(f"Error at {file.name} on {name}")
                    print(e)

            with open(file, 'w') as w:
                json.dump(data, w)
    except Exception as e:
        print(e)
    finally:
        driver.quit()


def get_creature_images():
    driver = get_driver()
    try:
        sleep(3)
        creatures_folder = Path.cwd() / Path('creatures')
        for file in creatures_folder.iterdir():
            data = {}
            with open(file, 'r') as r:
                data = json.load(r)
            iterlinks = iter_web_links(file)
            for name, link in iterlinks:
                driver.get(link)
                sleep(0.5)
                try:
                    if "brute" in name.lower():
                        continue
                    img = get_dino_img(driver, name)
                    for i in range(len(data)):
                        if name in data[i]['name']:
                            data[i]['images'] = img
                    for url in img:
                        content = requests.get(url).content
                        save_image(content, url, name, file, creatures_folder)
                except Exception as e:
                    print(f"Error at {file.name} on {name}")
                    print(e)

            with open(file, 'w') as w:
                json.dump(data, w)
    except Exception as e:
        print(e)
    finally:
        driver.quit()


def get_items_data():
    driver = get_driver()
    try:
        sleep(1)
        items_folder = Path.cwd() / Path('items')
        tools = items_folder / Path("xd.json")
        recipes = items_folder / Path("recipes.json")
        for file in [tools, recipes]:
            if file.is_dir():
                continue
            data = {}
            with open(file, 'r') as r:
                data = json.load(r)
            iterlinks = iter_web_links(file)
            for name, link in iterlinks:
                driver.get(link)
                sleep(0.5)
                try:
                    img = get_item_img(driver, name)
                    short_desc = get_short_description(driver)
                    if type(img) == str:
                        content = requests.get(img).content
                        save_image(content, img, name, file, items_folder)
                    elif type(img) == list:
                        for url in img:
                            content = requests.get(url).content
                            save_image(content, url, name, file, items_folder)
                    for i in range(len(data)):
                        if name in data[i]['name']:
                            data[i]['image'] = img
                            data[i]['short_description'] = short_desc
                except Exception as e:
                    print(f"Error at {file.name} on {name}")
                    print(e)

            with open(file, 'w') as w:
                json.dump(data, w)
    except Exception as e:
        print(e)
    finally:
        driver.quit()


if __name__ == "__main__":
    get_items_data()
    # "Archer Flex" Emote
    # "Bicep Smooch" Emote
    # "Dance" Emote
    # "Zombie" Emote




class Recipes:
    NOTE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_Note.PrimalItem_Note'"
    NOTES_ON_ROCKWELL_RECIPES = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_Measurements.PrimalItem_RecipeNote_Measurements'"
    ROCKWELL_RECIPES_BATTLE_TARTARE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_BattleTartare.PrimalItem_RecipeNote_BattleTartare'"
    ROCKWELL_RECIPES_CALIEN_SOUP = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_CalienSoup.PrimalItem_RecipeNote_CalienSoup'"
    ROCKWELL_RECIPES_DECORATIVE_COLORING = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_Dye.PrimalItem_RecipeNote_Dye'"
    ROCKWELL_RECIPES_EGG_BASED_KIBBLE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_Kibble.PrimalItem_RecipeNote_Kibble'"
    ROCKWELL_RECIPES_ENDURO_STEW = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_EnduroStew.PrimalItem_RecipeNote_EnduroStew'"
    ROCKWELL_RECIPES_ENERGY_BREW = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_StaminaSoup.PrimalItem_RecipeNote_StaminaSoup'"
    ROCKWELL_RECIPES_FOCAL_CHILI = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_FocalChili.PrimalItem_RecipeNote_FocalChili'"
    ROCKWELL_RECIPES_FRIA_CURRY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_FriaCurry.PrimalItem_RecipeNote_FriaCurry'"
    ROCKWELL_RECIPES_LAZARUS_CHOWDER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_LazarusChowder.PrimalItem_RecipeNote_LazarusChowder'"
    ROCKWELL_RECIPES_MEAT_JERKY = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_Jerky.PrimalItem_RecipeNote_Jerky'"
    ROCKWELL_RECIPES_MEDICAL_BREW = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_HealSoup.PrimalItem_RecipeNote_HealSoup'"
    ROCKWELL_RECIPES_MINDWIPE_TONIC = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_RespecSoup.PrimalItem_RecipeNote_RespecSoup'"
    ROCKWELL_RECIPES_SHADOW_STEAK_SAUTE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Notes/PrimalItem_RecipeNote_ShadowSteak.PrimalItem_RecipeNote_ShadowSteak'"



class Farming:
    SMALL_ANIMAL_FECES = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Consumables/PrimalItemConsumable_DinoPoopSmall.PrimalItemConsumable_DinoPoopSmall'"
    HUMAN_FECES = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Consumables/PrimalItemConsumable_HumanPoop.PrimalItemConsumable_HumanPoop'"
    MEDIUM_ANIMAL_FECES = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Consumables/PrimalItemConsumable_DinoPoopMedium.PrimalItemConsumable_DinoPoopMedium'"
    LARGE_ANIMAL_FECES = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Consumables/PrimalItemConsumable_DinoPoopLarge.PrimalItemConsumable_DinoPoopLarge'"
    MASSIVE_ANIMAL_FECES = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Consumables/PrimalItemConsumable_DinoPoopMassive.PrimalItemConsumable_DinoPoopMassive'"
    SNOW_OWL_PELLET = "Blueprint'/Game/Extinction/Dinos/Owl/Pellet/PrimalItemConsumable_OwlPellet.PrimalItemConsumable_OwlPellet'"


class Island:
    CAVE_BEACON_1 = "SupplyCrate_Cave_QualityTier1_C"
    CAVE_BEACON_2 = "SupplyCrate_Cave_QualityTier2_C"
    CAVE_BEACON_3 = "SupplyCrate_Cave_QualityTier3_C"
    CAVE_BEACON_4 = "SupplyCrate_Cave_QualityTier4_C"
    SWAMP_CAVE_LOOT_CRATE_BLUE = "SupplyCrate_SwampCaveTier1_C"
    SWAMP_CAVE_LOOT_CRATE_YELLOW = "SupplyCrate_SwampCaveTier2_C"
    SWAMP_CAVE_LOOT_CRATE_RED = "SupplyCrate_SwampCaveTier3_C"
    ICE_CAVE_LOOT_CRATE_BLUE = "SupplyCrate_IceCaveTier1_C"
    ICE_CAVE_LOOT_CRATE_YELLOW = "SupplyCrate_IceCaveTier2_C"
    ICE_CAVE_LOOT_CRATE_RED = "SupplyCrate_IceCaveTier3_C"
    UNDERWATER_CAVES_LOOT_CRATE_BLUE = "SupplyCrate_UnderwaterCaveTier1_C"
    UNDERWATER_CAVES_LOOT_CRATE_YELLOW = "SupplyCrate_UnderwaterCaveTier2_C"
    UNDERWATER_CAVES_LOOT_CRATE_RED = "SupplyCrate_UnderwaterCaveTier3_C"
    DEEP_SEA_LOOT_CRATE = "SupplyCrate_OceanInstant_C"
    SUPPLYCRATE_OCEANINSTANT_HIGH_C = ""
    WHITE_BEACON = "SupplyCrate_Level03_C"
    WHITE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level03_Double_C"
    GREEN_BEACON = "SupplyCrate_Level15_C"
    GREEN_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level15_Double_C"
    BLUE_BEACON = "SupplyCrate_Level25_C"
    BLUE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level25_Double_C"
    PURPLE_BEACON = "SupplyCrate_Level35_C"
    PURPLE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level35_Double_C"
    YELLOW_BEACON = "SupplyCrate_Level45_C"
    YELLOW_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level45_Double_C"
    RED_BEACON = "SupplyCrate_Level60_C"
    RED_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level60_Double_C"
    BEAVER_DAM = "BeaverDam_C"
    GIANT_BEAVER_DAM_LOGS = "SupplyCrateBaseBP_Instantaneous_DamLogs_Child_C"
    GIANT_BEAVER_DAM = "SupplyCrateBaseBP_Instantaneous_DenLogs_Child2_C"



class Center:
    CAVE_BEACON_1 = "SupplyCrate_Cave_QualityTier1_C"
    CAVE_BEACON_2 = "SupplyCrate_Cave_QualityTier2_C"
    CAVE_BEACON_3 = "SupplyCrate_Cave_QualityTier3_C"
    CAVE_BEACON_4 = "SupplyCrate_Cave_QualityTier4_C"
    SWAMP_CAVE_LOOT_CRATE_BLUE = "SupplyCrate_SwampCaveTier1_C"
    SWAMP_CAVE_LOOT_CRATE_YELLOW = "SupplyCrate_SwampCaveTier2_C"
    SWAMP_CAVE_LOOT_CRATE_RED = "SupplyCrate_SwampCaveTier3_C"
    ICE_CAVE_LOOT_CRATE_BLUE = "SupplyCrate_IceCaveTier1_C"
    ICE_CAVE_LOOT_CRATE_YELLOW = "SupplyCrate_IceCaveTier2_C"
    ICE_CAVE_LOOT_CRATE_RED = "SupplyCrate_IceCaveTier3_C"
    UNDERWATER_CAVES_LOOT_CRATE_BLUE = "SupplyCrate_UnderwaterCaveTier1_C"
    UNDERWATER_CAVES_LOOT_CRATE_YELLOW = "SupplyCrate_UnderwaterCaveTier2_C"
    UNDERWATER_CAVES_LOOT_CRATE_RED = "SupplyCrate_UnderwaterCaveTier3_C"
    DEEP_SEA_LOOT_CRATE = "SupplyCrate_OceanInstant_C"
    SUPPLYCRATE_OCEANINSTANT_HIGH_C = ""
    WHITE_BEACON = "SupplyCrate_Level03_C"
    WHITE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level03_Double_C"
    GREEN_BEACON = "SupplyCrate_Level15_C"
    GREEN_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level15_Double_C"
    BLUE_BEACON = "SupplyCrate_Level25_C"
    BLUE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level25_Double_C"
    PURPLE_BEACON = "SupplyCrate_Level35_C"
    PURPLE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level35_Double_C"
    YELLOW_BEACON = "SupplyCrate_Level45_C"
    YELLOW_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level45_Double_C"
    RED_BEACON = "SupplyCrate_Level60_C"
    RED_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level60_Double_C"
    BEAVER_DAM = "BeaverDam_C"
    GIANT_BEAVER_DAM_LOGS = "SupplyCrateBaseBP_Instantaneous_DamLogs_Child_C"
    GIANT_BEAVER_DAM = "SupplyCrateBaseBP_Instantaneous_DenLogs_Child2_C"


class Valguero:
    pass


class Ragnarok:
    TREASURE_CHEST = "SupplyCrate_Chest_Treasure_JacksonL_C"
    CAVE_BEACON_1 = "SupplyCrate_Cave_QualityTier1_C"
    CAVE_BEACON_2 = "SupplyCrate_Cave_QualityTier2_C"
    CAVE_BEACON_3 = "SupplyCrate_Cave_QualityTier3_C"
    CAVE_BEACON_4 = "SupplyCrate_Cave_QualityTier4_C"
    SWAMP_CAVE_LOOT_CRATE_BLUE = "SupplyCrate_SwampCaveTier1_C"
    SWAMP_CAVE_LOOT_CRATE_YELLOW = "SupplyCrate_SwampCaveTier2_C"
    SWAMP_CAVE_LOOT_CRATE_RED = "SupplyCrate_SwampCaveTier3_C"
    ICE_CAVE_LOOT_CRATE_BLUE = "SupplyCrate_IceCaveTier1_C"
    ICE_CAVE_LOOT_CRATE_YELLOW = "SupplyCrate_IceCaveTier2_C"
    ICE_CAVE_LOOT_CRATE_RED = "SupplyCrate_IceCaveTier3_C"
    UNDERWATER_CAVES_LOOT_CRATE_BLUE = "SupplyCrate_UnderwaterCaveTier1_C"
    UNDERWATER_CAVES_LOOT_CRATE_YELLOW = "SupplyCrate_UnderwaterCaveTier2_C"
    UNDERWATER_CAVES_LOOT_CRATE_RED = "SupplyCrate_UnderwaterCaveTier3_C"
    DEEP_SEA_LOOT_CRATE = "SupplyCrate_OceanInstant_C"
    SUPPLYCRATE_OCEANINSTANT_HIGH_C = ""
    WHITE_BEACON = "SupplyCrate_Level03_C"
    WHITE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level03_Double_C"
    GREEN_BEACON = "SupplyCrate_Level15_C"
    GREEN_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level15_Double_C"
    BLUE_BEACON = "SupplyCrate_Level25_C"
    BLUE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level25_Double_C"
    PURPLE_BEACON = "SupplyCrate_Level35_C"
    PURPLE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level35_Double_C"
    YELLOW_BEACON = "SupplyCrate_Level45_C"
    YELLOW_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level45_Double_C"
    RED_BEACON = "SupplyCrate_Level60_C"
    RED_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level60_Double_C"
    CAVE_BEACON_1 = "SupplyCrate_Cave_QualityTier1_ScorchedEarth_C"
    CAVE_BEACON_2 = "SupplyCrate_Cave_QualityTier2_ScorchedEarth_C"
    CAVE_BEACON_3 = "SupplyCrate_Cave_QualityTier3_ScorchedEarth_C"
    WHITE_BEACON = "SupplyCrate_Level03_ScorchedEarth_C"
    WHITE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level03_Double_ScorchedEarth_C"
    GREEN_BEACON = "SupplyCrate_Level15_ScorchedEarth_C"
    GREEN_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level15_Double_ScorchedEarth_C"
    BLUE_BEACON = "SupplyCrate_Level30_ScorchedEarth_C"
    BLUE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level30_Double_ScorchedEarth_C"
    PURPLE_BEACON = "SupplyCrate_Level45_ScorchedEarth_C"
    PURPLE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level45_Double_ScorchedEarth_C"
    YELLOW_BEACON = "SupplyCrate_Level55_ScorchedEarth_C"
    YELLOW_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level55_Double_ScorchedEarth_C"
    RED_BEACON = "SupplyCrate_Level70_ScorchedEarth_C"
    RED_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level70_Double_ScorchedEarth_C"


class Extinction:
    CAVE_LOOT_CRATE_BLUE = "SupplyCrate_Cave_QualityTier1_EX_C"
    CAVE_LOOT_CRATE_YELLOW = "SupplyCrate_Cave_QualityTier2_EX_C"
    CAVE_LOOT_CRATE_RED = "SupplyCrate_Cave_QualityTier3_EX_C"
    ORBITAL_SUPPLY_DROP_BLUE = "SupplyCrate_Base_Horde_Easy_C"
    ORBITAL_SUPPLY_DROP_YELLOW = "SupplyCrate_Base_Horde_Medium_C"
    ORBITAL_SUPPLY_DROP_RED = "SupplyCrate_Base_Horde_Hard_C"
    ORBITAL_SUPPLY_DROP_PURPLE = "SupplyCrate_Base_Horde_Legendary_C"
    CORRUPT_ELEMENT_NODE_EASY = "ElementNode_Easy_Horde_C"
    CORRUPT_ELEMENT_NODE_HARD = "ElementNode_Hard_Horde_C"
    CORRUPT_ELEMENT_NODE_MEDIUM = "ElementNode_Medium_Horde_C"
    CORRUPT_ELEMENT_NODE_KINGKAIJU = "KingKaiju_ElementNode_C"
    ARTIFACT_CONTAINER_CHAOS = "ArtifactCrate_Desert_Kaiju_EX_C"
    ARTIFACT_CONTAINER_GROWTH = "ArtifactCrate_ForestKaiju_EX_C"
    ARTIFACT_CONTAINER_VOID = "ArtifactCrate_IceKaiju_EX_C"
    KING_TITAN_ALPHA = "ArtifactCrate_KingKaiju_Alpha_EX_C"
    KING_TITAN_BETA = "ArtifactCrate_KingKaiju_Beta_EX_C"
    KING_TITAN_GAMMA = "ArtifactCrate_KingKaiju_EX_C"


class Aberration:
    WHITE_CRATE = "SupplyCrate_Cave_Aberration_Level10_C"
    WHITE_CRATE_DOUBLE = "SupplyCrate_Cave_Aberration_Level10_Double_C"
    GREEN_CRATE = "SupplyCrate_Cave_Aberration_Level25_C"
    GREEN_CRATE_DOUBLE = "SupplyCrate_Cave_Aberration_Level25_Double_C"
    BLUE_CRATE = "SupplyCrate_Cave_Aberration_Level35_C"
    BLUE_CRATE_DOUBLE = "SupplyCrate_Cave_Aberration_Level35_Double_C"
    PURPLE_CRATE = "SupplyCrate_Cave_Aberration_Level50_C"
    PURPLE_CRATE_DOUBLE = "SupplyCrate_Cave_Aberration_Level50_Double_C"
    YELLOW_CRATE = "SupplyCrate_Cave_Aberration_Level65_C"
    YELLOW_CRATE_DOUBLE = "SupplyCrate_Cave_Aberration_Level65_Double_C"
    RED_CRATE = "SupplyCrate_Cave_Aberration_Level80_C"
    RED_CRATE_DOUBLE = "SupplyCrate_Cave_Aberration_Level80_Double_C"
    BLUE_DUNGEON_CRATE = "SupplyCrate_Dungeon_Aberration_Level35_C"
    PURPLE_DUNGEON_CRATE = "SupplyCrate_Dungeon_Aberration_Level50_C"
    YELLOW_DUNGEON_CRATE = "SupplyCrate_Dungeon_Aberration_Level65_C"
    RED_DUNGEON_CRATE = "SupplyCrate_Dungeon_Aberration_Level80_C"
    BLUE_SURFACE_BEACON = "SupplyCrate_Level35_Aberrant_Surface_C"
    BLUE_SURFACE_BEACON_DOUBLE = "SupplyCrate_Level35_Aberrant_Surface_Double_C"
    PURPLE_SURFACE_BEACON = "SupplyCrate_Level50_Aberrant_Surface_C"
    PURPLE_SURFACE_BEACON_DOUBLE = "SupplyCrate_Level50_Aberrant_Surface_Double_C"
    YELLOW_SURFACE_BEACON = "SupplyCrate_Level65_Aberrant_Surface_C"
    YELLOW_SURFACE_BEACON_DOUBLE = "SupplyCrate_Level65_Aberrant_Surface_Double_C"
    RED_SURFACE_BEACON = "SupplyCrate_Level80_Aberrant_Surface_C"
    RED_SURFACE_BEACON_DOUBLE = "SupplyCrate_Level80_Aberrant_Surface_Double_C"
    ARTIFACT_CONTAINER_DEPTHS = "ArtifactCrate_AB_C"
    ARTIFACT_CONTAINER_SHADOWS = "ArtifactCrate_2_AB_C"
    ARTIFACT_CONTAINER_STALKER = "ArtifactCrate_3_AB_C"
    ARTIFACT_CONTAINER_LOST = "ArtifactCrate_4_AB_C"


class Scortched:
    CAVE_BEACON_1 = "SupplyCrate_Cave_QualityTier1_ScorchedEarth_C"
    CAVE_BEACON_2 = "SupplyCrate_Cave_QualityTier2_ScorchedEarth_C"
    CAVE_BEACON_3 = "SupplyCrate_Cave_QualityTier3_ScorchedEarth_C"
    WHITE_BEACON = "SupplyCrate_Level03_ScorchedEarth_C"
    WHITE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level03_Double_ScorchedEarth_C"
    GREEN_BEACON = "SupplyCrate_Level15_ScorchedEarth_C"
    GREEN_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level15_Double_ScorchedEarth_C"
    BLUE_BEACON = "SupplyCrate_Level30_ScorchedEarth_C"
    BLUE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level30_Double_ScorchedEarth_C"
    PURPLE_BEACON = "SupplyCrate_Level45_ScorchedEarth_C"
    PURPLE_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level45_Double_ScorchedEarth_C"
    YELLOW_BEACON = "SupplyCrate_Level55_ScorchedEarth_C"
    YELLOW_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level55_Double_ScorchedEarth_C"
    RED_BEACON = "SupplyCrate_Level70_ScorchedEarth_C"
    RED_BEACON_DOUBLE_ITEMS = "SupplyCrate_Level70_Double_ScorchedEarth_C"
    ARTIFACT_CONTAINER_DESTROYER = "ArtifactCrate_SE_C"
    ARTIFACT_CONTAINER_GATEKEEPER = "ArtifactCrate_2_SE_C"
    ARTIFACT_CONTAINER_CRAG = "ArtifactCrate_3_SE_C"


class Genesis2:
    AMBERGRIS_CRATE = "SupplyCrate_Space_01_Ambergris_C"
    CRYSTAL_CRATE = "SupplyCrate_Space_02_Crystal_C"
    SULFUR_CRATE = "SupplyCrate_Space_03_Sulfur_C"
    ELEMENTSHARDS_CRATE = "SupplyCrate_Space_04_Elementshards_C"
    OBSIDIAN_CRATE = "SupplyCrate_Space_05_Obsidian_C"
    OIL_CRATE = "SupplyCrate_Space_06_Oil_C"
    ELEMENTDUST_CRATE = "SupplyCrate_Space_07_Elementdust_C"
    BLACKPEARLS_CRATE = "SupplyCrate_Space_08_Blackpearls_C"


class Genesis1:
    pass




class Resources:
    ABSORBENT_SUBSTRATE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_SubstrateAbsorbent.PrimalItemResource_SubstrateAbsorbent'"
    ACHATINA_PASTE = "Blueprint'/Game/PrimalEarth/Dinos/Achatina/PrimalItemResource_SnailPaste.PrimalItemResource_SnailPaste'"
    AMBERGRIS = "Blueprint'/Game/Genesis/Dinos/SpaceWhale/PrimalItemResource_Ambergris.PrimalItemResource_Ambergris'"
    AMMONITE_BILE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_AmmoniteBlood.PrimalItemResource_AmmoniteBlood'"
    ANGLERGEL = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_AnglerGel.PrimalItemResource_AnglerGel'"
    BLACK_PEARL = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_BlackPearl.PrimalItemResource_BlackPearl'"
    BLUE_CRYSTALIZED_SAP = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_BlueSap.PrimalItemResource_BlueSap'"
    BLUE_GEM = "Blueprint'/Game/Aberration/CoreBlueprints/Resources/PrimalItemResource_Gem_BioLum.PrimalItemResource_Gem_BioLum'"
    BLOOD_PACK = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Consumables/PrimalItemConsumable_BloodPack.PrimalItemConsumable_BloodPack'"
    CEMENTING_PASTE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ChitinPaste.PrimalItemResource_ChitinPaste'"
    CHARCOAL = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Charcoal.PrimalItemResource_Charcoal'"
    CHARGE_BATTERY = "Blueprint'/Game/Aberration/WeaponGlowStickCharge/PrimalItem_ChargeBattery.PrimalItem_ChargeBattery'"
    CHITIN_OR_KERATIN = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ChitinOrKeratin.PrimalItemResource_ChitinOrKeratin'"
    CHITIN = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Chitin.PrimalItemResource_Chitin'"
    CLAY = "Blueprint'/Game/ScorchedEarth/CoreBlueprints/Resources/PrimalItemResource_Clay.PrimalItemResource_Clay'"
    CONDENSED_GAS = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_CondensedGas.PrimalItemResource_CondensedGas'"
    CONGEALED_GAS_BALL = "Blueprint'/Game/Aberration/CoreBlueprints/Resources/PrimalItemResource_Gas.PrimalItemResource_Gas'"
    CORRUPTED_NODULE = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_CorruptedPolymer.PrimalItemResource_CorruptedPolymer'"
    CRAFTED_ELEMENT_DUST = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_ElementDustFromShards.PrimalItemResource_ElementDustFromShards'"
    CRYSTAL = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Crystal.PrimalItemResource_Crystal'"
    DEATHWORM_HORN = "Blueprint'/Game/ScorchedEarth/Dinos/Deathworm/PrimalItemResource_KeratinSpike.PrimalItemResource_KeratinSpike'"
    DERMIS = "Blueprint'/Game/Extinction/CoreBlueprints/Items/PrimalItem_TaxidermyDermis.PrimalItem_TaxidermyDermis'"
    DINOSAUR_BONE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ARKBone.PrimalItemResource_ARKBone'"
    ELECTRONICS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Electronics.PrimalItemResource_Electronics'"
    ELEMENT = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Element.PrimalItemResource_Element'"
    ELEMENT_DUST = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_ElementDust.PrimalItemResource_ElementDust'"
    ELEMENT_ORE = "Blueprint'/Game/Aberration/CoreBlueprints/Resources/PrimalItemResource_ElementOre.PrimalItemResource_ElementOre'"
    ELEMENT_SHARD = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_ElementShard.PrimalItemResource_ElementShard'"
    FERTILIZER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Consumables/PrimalItemConsumable_Fertilizer_Compost.PrimalItemConsumable_Fertilizer_Compost'"
    FIBER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Fibers.PrimalItemResource_Fibers'"
    FLINT = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Flint.PrimalItemResource_Flint'"
    FRAGMENTED_GREEN_GEM = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_FracturedGem.PrimalItemResource_FracturedGem'"
    FUNGAL_WOOD = "Blueprint'/Game/Aberration/CoreBlueprints/Resources/PrimalItemResource_FungalWood.PrimalItemResource_FungalWood'"
    CORRUPTED_WOOD = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_CorruptedWood.PrimalItemResource_CorruptedWood'"
    GASOLINE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Gasoline.PrimalItemResource_Gasoline'"
    GOLDEN_NUGGET = "Blueprint'/Game/Genesis/Missions/Retrieve/RetrieveItems/GoldenNugget/PrimalItemResource_GoldenNugget.PrimalItemResource_GoldenNugget'"
    GREEN_GEM = "Blueprint'/Game/Aberration/CoreBlueprints/Resources/PrimalItemResource_Gem_Fertile.PrimalItemResource_Gem_Fertile'"
    GUNPOWDER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Gunpowder.PrimalItemResource_Gunpowder'"
    HIDE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Hide.PrimalItemResource_Hide'"
    HIGH_QUALITY_POLLEN = "Blueprint'/Game/Genesis/Missions/Retrieve/RetrieveItems/HighQualityPollen/PrimalItemResource_HighQualityPollen.PrimalItemResource_HighQualityPollen'"
    HUMAN_HAIR = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Hair.PrimalItemResource_Hair'"
    KERATIN = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Keratin.PrimalItemResource_Keratin'"
    LEECH_BLOOD = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_LeechBlood.PrimalItemResource_LeechBlood'"
    LEECH_BLOOD_OR_HORNS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResourceGeneric_Curing.PrimalItemResourceGeneric_Curing'"
    METAL_INGOT = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_MetalIngot.PrimalItemResource_MetalIngot'"
    METAL = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Metal.PrimalItemResource_Metal'"
    MUTAGEL = "Blueprint'/Game/Genesis2/CoreBlueprints/Environment/Mutagen/PrimalItemConsumable_Mutagel.PrimalItemConsumable_Mutagel'"
    MUTAGEN = "Blueprint'/Game/Genesis2/CoreBlueprints/Environment/Mutagen/PrimalItemConsumable_Mutagen.PrimalItemConsumable_Mutagen'"
    NARCOTIC = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Consumables/PrimalItemConsumable_Narcotic.PrimalItemConsumable_Narcotic'"
    OBSIDIAN = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Obsidian.PrimalItemResource_Obsidian'"
    OIL = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Oil.PrimalItemResource_Oil'"
    OIL_TUSOTEUTHIS = "Blueprint'/Game/PrimalEarth/Dinos/Tusoteuthis/PrimalItemResource_SquidOil.PrimalItemResource_SquidOil'"
    ORGANIC_POLYMER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Polymer_Organic.PrimalItemResource_Polymer_Organic'"
    PELT = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Pelt.PrimalItemResource_Pelt'"
    POLYMER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Polymer.PrimalItemResource_Polymer'"
    PRIMAL_CRYSTAL = "Blueprint'/Game/PrimalEarth/Dinos/CrystalWyvern/CrystalResources/Primal/PrimalItemResource_Crystal_IslesPrimal.PrimalItemResource_Crystal_IslesPrimal'"
    PRESERVING_SALT = "Blueprint'/Game/ScorchedEarth/CoreBlueprints/Resources/PrimalItemResource_PreservingSalt.PrimalItemResource_PreservingSalt'"
    PROPELLANT = "Blueprint'/Game/ScorchedEarth/CoreBlueprints/Resources/PrimalItemResource_Propellant.PrimalItemResource_Propellant'"
    RARE_FLOWER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_RareFlower.PrimalItemResource_RareFlower'"
    RARE_MUSHROOM = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_RareMushroom.PrimalItemResource_RareMushroom'"
    RAW_SALT = "Blueprint'/Game/ScorchedEarth/CoreBlueprints/Resources/PrimalItemResource_RawSalt.PrimalItemResource_RawSalt'"
    RE_FERTILIZER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Consumables/BaseBPs/PrimalItemConsumableMiracleGro.PrimalItemConsumableMiracleGro'"
    RED_CRYSTALIZED_SAP = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_RedSap.PrimalItemResource_RedSap'"
    RED_GEM = "Blueprint'/Game/Aberration/CoreBlueprints/Resources/PrimalItemResource_Gem_Element.PrimalItemResource_Gem_Element'"
    SAND = "Blueprint'/Game/ScorchedEarth/CoreBlueprints/Resources/PrimalItemResource_Sand.PrimalItemResource_Sand'"
    SAP = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Sap.PrimalItemResource_Sap'"
    SCRAP_METAL = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_ScrapMetal.PrimalItemResource_ScrapMetal'"
    SCRAP_METAL_INGOT = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_ScrapMetalIngot.PrimalItemResource_ScrapMetalIngot'"
    SHELL_FRAGMENT = "Blueprint'/Game/Genesis/CoreBlueprints/Resources/PrimalItemResource_TurtleShell.PrimalItemResource_TurtleShell'"
    SILICA_PEARLS = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Silicon.PrimalItemResource_Silicon'"
    SILICATE = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_Silicate.PrimalItemResource_Silicate'"
    SILK = "Blueprint'/Game/ScorchedEarth/CoreBlueprints/Resources/PrimalItemResource_Silk.PrimalItemResource_Silk'"
    SPARKPOWDER = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Sparkpowder.PrimalItemResource_Sparkpowder'"
    STIMULANT = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Items/Consumables/PrimalItemConsumable_Stimulant.PrimalItemConsumable_Stimulant'"
    STONE = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Stone.PrimalItemResource_Stone'"
    SULFUR = "Blueprint'/Game/ScorchedEarth/CoreBlueprints/Resources/PrimalItemResource_Sulfur.PrimalItemResource_Sulfur'"
    THATCH = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Thatch.PrimalItemResource_Thatch'"
    UNSTABLE_ELEMENT = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_ElementRefined.PrimalItemResource_ElementRefined'"
    UNSTABLE_ELEMENT_SHARD = "Blueprint'/Game/Extinction/CoreBlueprints/Resources/PrimalItemResource_ShardRefined.PrimalItemResource_ShardRefined'"
    WOOD = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Wood.PrimalItemResource_Wood'"
    WOOL = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Wool.PrimalItemResource_Wool'"
    WOOLLY_RHINO_HORN = "Blueprint'/Game/PrimalEarth/CoreBlueprints/Resources/PrimalItemResource_Horn.PrimalItemResource_Horn'"